import logging
import sys
from datetime import datetime

from django.conf import settings
from django.db import transaction
from django.db.models import DateTimeField, Sum
from django.forms import model_to_dict

from isc_common.auth.models.user import User
from isc_common.fields.related import ForeignKeyProtect, ForeignKeySetNull
from isc_common.http.DSRequest import DSRequest
from isc_common.models.base_ref import Hierarcy
from isc_common.models.standard_colors import Standard_colors
from isc_common.number import DelProps
from kaf_pas.planing.models.operation_types import Operation_types
from kaf_pas.planing.models.operation_value import Operation_value
from kaf_pas.planing.models.operations import OperationsManager, OperationsQuerySet

logger = logging.getLogger(__name__)


class Production_order_valuesQuerySet(OperationsQuerySet):
    pass


class Production_order_valuesManager(OperationsManager):
    def createFromRequest(self, request, removed=None):
        from kaf_pas.planing.models.operations import Operations
        from kaf_pas.planing.models.production_order_detail import Production_order_detail
        from kaf_pas.ckk.models.ed_izm import Ed_izm

        request = DSRequest(request=request)
        data = request.get_data()
        opertype = request.get_operationtype()

        if not isinstance(data.get('child_ids'), list):
            raise Exception(f'Не выбраны производственные операции(ия).')

        if not isinstance(data.get('parent_id'), int):
            raise Exception(f'Не выбран заказ на производство.')

        if not isinstance(data.get('edizm_id'), int):
            raise Exception(f'Не указана еденница измерения.')

        childs = Production_order_detail.objects.filter(
            id__in=data.get('child_ids'),
            parent_id=data.get('parent_id'),
            opertype=settings.OPERS_TYPES_STACK.PRODUCTION_DETAIL_TASK,
            value__props=~Operation_value.props.perone
        )
        value = data.get('value')

        edizm = Ed_izm.objects.get(id=data.get('edizm_id'))

        with transaction.atomic():
            for parent in Operations.objects.select_for_update().filter(id=data.get('parent_id')):
                res = Production_order_valuesManager.makeAll(
                    parent=parent,
                    childs=childs,
                    value=value,
                    opertype=opertype,
                    edizm=edizm,
                    user=request.user
                )

            return res

    # IUD - Insert/Update/Delete
    @staticmethod
    def makeAll(parent, childs, value, opertype, edizm, user):
        from kaf_pas.production.models.launch_item_refs import Launch_item_refs
        from kaf_pas.accounting.models.buffers import BuffersManager
        from kaf_pas.planing.models.operations import Operations
        from kaf_pas.planing.models.operation_refs import Operation_refs
        from kaf_pas.planing.models.operation_item import Operation_item
        from kaf_pas.planing.models.operation_launches import Operation_launches
        from kaf_pas.planing.models.production_order_detail import Production_order_detail

        # parent - заказ на производство,
        # childs - производственные операции,
        # value количетвенная операция данной производственной операции
        # opertype - тип REST операции "add", "update" etc

        res = []
        settings.LOCKS.acquire('Production_order_valuesManager.makeAll')
        if opertype == 'add':

            # Определяем позволяет ли остатки в операциях записать их как сделанные
            for child in childs:
                tail = round(child.value_start - child.value_made, 4)
                if value > tail:
                    raise Exception(f'По операции {child.production_operation.full_name} превышен остаток {tail}, а затребовано {round(value, 4)}.')
                else:
                    operations = Operations.objects.create(
                        opertype=settings.OPERS_TYPES_STACK.MADE_OPERATIONS_TASK,
                        date=datetime.now(),
                        status=settings.OPERS_TYPES_STACK.MADE_OPERATIONS_STATUSES.get('new'),
                        creator=user
                    )
                    logger.debug(f'Created operations: {operations}')

                    operation_refs = Operation_refs.objects.create(parent_id=child.id, child=operations)
                    logger.debug(f'Created operation_refs: {operation_refs}')

                    operation_value = Operation_value.objects.create(
                        operation=operations,
                        edizm=edizm,
                        value=value,
                    )
                    logger.debug(f'Created operation_value: {operation_value}')
                    res.append(dict(
                        id=operations.id,
                        date=operations.date,
                        value=operation_value.value,
                        edizm_id=edizm.id,
                        edizm__name=edizm.name,
                        creator__short_name=operations.creator.get_short_name
                    ))

            # Просматриваем все операции на предмет выявления максимального выпуска
            # min_value минимальное возможное оформление выпуска

            def values_not_used_in_release(id):
                return Production_order_values.objects.filter(
                    parent_id=id,
                    value__props=~Operation_value.props.used_in_release,
                    opertype=settings.OPERS_TYPES_STACK.MADE_OPERATIONS_TASK
                )

            min_qty_release = sys.maxsize
            production_order_detail_arr = []
            for production_order_detail in Production_order_detail.objects.filter(
                    parent_id=parent.id,
                    opertype=settings.OPERS_TYPES_STACK.PRODUCTION_DETAIL_TASK):

                values_not_used_in_release_sum = values_not_used_in_release(id=production_order_detail.id).aggregate(Sum('value__value')).get('value__value__sum', 0)
                if values_not_used_in_release_sum > 0:
                    min_qty_release_prev = values_not_used_in_release_sum // production_order_detail.value_per_one
                    if min_qty_release_prev == 0:
                        return res
                    else:
                        if min_qty_release > min_qty_release_prev:
                            min_qty_release = min_qty_release_prev
                        production_order_detail_arr.append(production_order_detail)

            # Проверка на наличее в буферах комплектующих
            items = []
            for item in Launch_item_refs.objects.filter(
                    launch=Operation_launches.objects.get(operation=parent).launch,
                    parent=Operation_item.objects.get(operation=parent).item
            ):
                qty_exists = BuffersManager.getValue(item=item.child)
                summa_tot = min_qty_release * item.qty_per_one
                if qty_exists == None:
                    return res
                elif qty_exists < summa_tot:
                    min_qty_release_prev = qty_exists // item.qty_per_one
                    if min_qty_release_prev > 0:
                        min_qty_release = min_qty_release_prev
                        items.append((item, min_qty_release * item.qty_per_one))
                    else:
                        return res
                elif qty_exists > summa_tot:
                    items.append((item, summa_tot))

            for item, value in items:
                # Операция с плюсом по тов позиции выпуска
                res = Operations.objects.create(
                    opertype=settings.OPERS_TYPES_STACK.RELEASE_TASK_MINUS,
                    date=datetime.now(),
                    status=settings.OPERS_TYPES_STACK.RELEASE_TASK_MINUS.get('new'),
                    creator=user
                )

                Operation_item.objects.create(
                    operation=res,
                    item=parent.item,
                    item_full_name=item.full_name,
                    item_full_name_obj=item.full_name_obj
                )

                Operation_refs.objects.create(parent=parent, child=res)
                Operation_value.objects.create(operation=res, edizm=edizm, value=value)

            releases_det_operations = []
            for production_order_detail in production_order_detail_arr:
                total_value = production_order_detail.value * min_qty_release
                for not_used_in_release_used in values_not_used_in_release(id).order_by('value'):
                    if not_used_in_release_used.value <= total_value:
                        operation_value, _ = Operation_value.objects.update_or_create(
                            id=not_used_in_release_used.value.id,
                            defaults=dict(props=Operation_value.props.used_in_release))

                        releases_det_operations.append(operation_value.operation)
                        total_value -= not_used_in_release_used.value
                    else:
                        operation_value = Operation_value.objects.get(id=not_used_in_release_used.value.id)
                        operation_value.props = Operation_value.props.used_in_release
                        operation_value.value = total_value
                        operation_value.save()
                        releases_det_operations.append(operation_value.operation)

                        operation = Operations.objects.get(id=operation_value.operation)
                        res = Operations.objects.create(
                            opertype=operation.opertype,
                            date=operation.date,
                            status=operation.status,
                            creator=operation.user
                        )

                        Operation_refs.objects.create(parent=operation.child, child=res)
                        Operation_value.objects.create(
                            operation=res,
                            edizm=edizm,
                            value=not_used_in_release_used.valu - total_value,
                        )

            first_step = True
            parent_release = None
            for releases_det_operation in releases_det_operations:
                if first_step:
                    parent_release = Operations.objects.create(
                        opertype=settings.OPERS_TYPES_STACK.RELEASE_TASK_PLUS,
                        date=datetime.now(),
                        status=settings.OPERS_TYPES_STACK.RELEASE_TASK_PLUS.get('new'),
                        creator=user
                    )
                    Operation_refs.objects.create(parent=parent, child=parent_release)
                    Operation_value.objects.create(operation=parent_release, edizm=edizm, value=min_qty_release, props=Operation_value.props.used_in_release)

                    Operation_item.objects.create(
                        operation=parent_release,
                        item=parent.item,
                        item_full_name=parent.item.full_name,
                        item_full_name_obj=parent.item.full_name_obj
                    )
                    first_step = False

                Operation_refs.objects.create(parent=parent_release, child=releases_det_operation)
        settings.LOCKS.release('Production_order_valuesManager.makeAll')

        return res

    def deleteFromRequest(self, request, removed=None):
        from kaf_pas.planing.models.operations import Operations
        from kaf_pas.planing.models.operation_refs import Operation_refs

        request = DSRequest(request=request)
        res = 0

        data = request.get_data()
        _transaction = data.get('transaction')
        ids = []
        if _transaction != None:
            operations = _transaction.get('operations')
            ids = [item.get('data').get('id') for item in operations]
        else:
            ids.append(data.get('id'))

        with transaction.atomic():
            for opeartion in Operations.objects.select_for_update().filter(id__in=ids):
                Operation_value.objects.filter(operation=opeartion).delete()
                Operation_refs.objects.filter(child=opeartion).delete()
                opeartion.delete()
        return res

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'color__color': record.color.color if record.color else None,
            'color__name': record.color.name if record.color else None,
            'color_id': record.color.id if record.color else None,
            'creator__short_name': record.creator.get_short_name,
            'creator_id': record.creator.id,
            'date': record.date,
            'edizm__code': record.value.edizm.code if record.value and record.value.edizm else None,
            'edizm__name': record.value.edizm.name if record.value and record.value.edizm else None,
            'edizm_id': record.value.edizm.id if record.value and record.value.edizm else None,
            'value': record.value.value,
            'used_in_release': record.value.props.used_in_release,
        }
        return DelProps(res)

    def get_queryset(self):
        return Production_order_valuesQuerySet(self.model, using=self._db)


class Production_order_values(Hierarcy):
    color = ForeignKeyProtect(Standard_colors, null=True, blank=True)
    opertype = ForeignKeyProtect(Operation_types)
    creator = ForeignKeyProtect(User, default=None)
    date = DateTimeField(default=None)
    value = ForeignKeySetNull(Operation_value, null=True, blank=True, related_name='planing_Production_order_values_view')

    objects = Production_order_valuesManager()

    def __str__(self):
        return super().__str__()

    def __repr__(self):
        return self.__str__()

    class Meta:
        managed = False
        db_table = 'planing_operations_values_view'
        verbose_name = 'Списания'



# Generated by Django 2.2.8 on 2019-12-07 18:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0042_auto_20191207_1820'),
    ]

    operations = [
        migrations.AddField(
            model_name='operations',
            name='level',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-28 21:05

import django.db.models.deletion
from django.db import migrations

import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0099_auto_20191028_2105'),
    ]

    operations = [
        migrations.AlterField(
            model_name='dogovors',
            name='customer',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='sales.Customer'),
        ),
        migrations.AlterField(
            model_name='dogovors',
            name='status',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='sales.StatusDogovor'),
        ),
        migrations.AlterField(
            model_name='dogovors',
            name='type',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='sales.Dogovor_types'),
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-30 17:45

from django.db import migrations
import isc_common.fields.code_field


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0149_remove_status_code_owner'),
    ]

    operations = [
        migrations.AddField(
            model_name='status',
            name='code_owner',
            field=isc_common.fields.code_field.CodeField(blank=True, null=True),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-29 12:23

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0034_resource'),
    ]

    operations = [
        migrations.CreateModel(
            name='Operation_resources',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('operation', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='production.Operations')),
                ('resource', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='production.Resource')),
            ],
            options={
                'verbose_name': 'Кросс таблица',
            },
        ),
    ]



# Generated by Django 2.2.3 on 2019-07-28 19:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0073_remove_document_attr_cross_props'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='pathes',
            name='prefix',
        ),
        migrations.AddField(
            model_name='pathes',
            name='drive',
            field=models.CharField(blank=True, max_length=10, null=True, verbose_name='Диск'),
        ),
        migrations.AlterModelTable(
            name='documents_view',
            table='kd_documents_view',
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-13 13:34

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0095_auto_20191213_1332'),
    ]

    operations = [
    ]



from django.urls import path

from kaf_pas.sales.views import dogovor_types

urlpatterns = [

    path('Dogovor_types/Fetch/', dogovor_types.Dogovor_types_Fetch),
    path('Dogovor_types/Add', dogovor_types.Dogovor_types_Add),
    path('Dogovor_types/Update', dogovor_types.Dogovor_types_Update),
    path('Dogovor_types/Remove', dogovor_types.Dogovor_types_Remove),
    path('Dogovor_types/Lookup/', dogovor_types.Dogovor_types_Lookup),
    path('Dogovor_types/Info/', dogovor_types.Dogovor_types_Info),
    path('Dogovor_types/Copy', dogovor_types.Dogovor_types_Copy),

]



import logging

from kaf_pas.kd.models.documents import Documents, DocumentManager, DocumentQuerySet

logger = logging.getLogger(__name__)


class CdwQuerySet(DocumentQuerySet):
    ...


class CdwManager(DocumentManager):
    def get_queryset(self):
        return CdwQuerySet(self.model, using=self._db).filter(attr_type__code='CDW')


class Cdws(Documents):
    objects = CdwManager()

    class Meta:
        verbose_name = 'Чертежи'
        proxy = True



from django.urls import path

from kaf_pas.planing.views import write_off_items

urlpatterns = [

    path('Write_off_items/Fetch/', write_off_items.Write_off_items_Fetch),
    path('Write_off_items/Add', write_off_items.Write_off_items_Add),
    path('Write_off_items/Update', write_off_items.Write_off_items_Update),
    path('Write_off_items/Remove', write_off_items.Write_off_items_Remove),
    path('Write_off_items/Lookup/', write_off_items.Write_off_items_Lookup),
    path('Write_off_items/Info/', write_off_items.Write_off_items_Info),
    path('Write_off_items/Copy', write_off_items.Write_off_items_Copy),

]



# Generated by Django 3.0.3 on 2020-02-16 19:08

from django.db import migrations
import isc_common.fields.code_field


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0183_ready_2_launch_detail_item_full_name_obj'),
    ]

    operations = [
        migrations.AddField(
            model_name='ready_2_launch_detail',
            name='mat_view_name',
            field=isc_common.fields.code_field.CodeField(blank=True, null=True),
        ),
    ]



# Generated by Django 3.0.2 on 2020-01-23 03:14

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0188_auto_20200122_1619'),
        ('kd', '0123_pathes_attr_type'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='document_attributes',
            unique_together={('attr_type', 'value_str', 'value_int')},
        ),
    ]



import logging

from bitfield import BitField

from isc_common.models.base_ref import BaseRef, BaseRefManager, BaseRefQuerySet

logger = logging.getLogger(__name__)


class Dogovor_typesQuerySet(BaseRefQuerySet):
    pass


class Dogovor_typesManager(BaseRefManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'code': record.code,
            'name': record.name,
            'description': record.description,

            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Dogovor_typesQuerySet(self.model, using=self._db)


class Dogovor_types(BaseRef):
    props = BitField(flags=(
        ('real', 'Реальный тим'),
    ), default=1, db_index=True)

    objects = Dogovor_typesManager()

    def __str__(self):
        return f"ID:{self.id}, code: {self.code}, name: {self.name}, description: {self.description}"

    class Meta:
        verbose_name = 'Типы договорных документов'



# Generated by Django 2.2.6 on 2019-10-14 13:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0082_auto_20191011_1650'),
    ]

    operations = [
        migrations.AddField(
            model_name='document_attributes',
            name='value_int',
            field=models.PositiveIntegerField(blank=True, db_index=True, null=True, verbose_name='Значение атрибута'),
        ),
    ]



# Generated by Django 2.2.3 on 2019-07-20 06:52

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0065_documents_history'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Documents_history',
        ),
    ]



from isc_common.http.DSRequest import DSRequest
from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.planing.models.operation_item import Operation_item, Operation_itemManager
from kaf_pas.planing.models.operations_view import Operations_view, Operations_viewManager


@JsonResponseWithException()
def Operation_item_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Operation_item.objects.
                filter().
                get_range_rows1(
                request=request,
                function=Operation_itemManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Operation_item_Fetch1(request):
    _request = DSRequest(request=request)
    data = _request.get_data()
    #
    # launch_id = data.get('launch_id')
    # operation_level_id = data.get('level_id')
    # item_id = data.get('level_id')

    return JsonResponse(
        DSResponse(
            request=request,
            data=Operations_view.objects.
                filter().
                distinct('item_full_name').
                get_range_rows1(
                request=request,
                function=Operations_viewManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Operation_item_Add(request):
    return JsonResponse(DSResponseAdd(data=Operation_item.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Operation_item_Update(request):
    return JsonResponse(DSResponseUpdate(data=Operation_item.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Operation_item_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Operation_item.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Operation_item_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Operation_item.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Operation_item_Info(request):
    return JsonResponse(DSResponse(request=request, data=Operation_item.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Operation_item_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Operation_item.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 2.2.1 on 2019-05-28 17:10

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0041_auto_20190528_1709'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='precent',
            unique_together={('code', 'type')},
        ),
    ]



# Generated by Django 2.2.7 on 2019-11-22 09:20

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0160_ed_izm_props'),
    ]

    operations = [
        migrations.AlterField(
            model_name='ed_izm',
            name='props',
            field=bitfield.models.BitField((('root_timevalue', 'Базовая временная единица'),), db_index=True, default=0),
        ),
    ]



from django.urls import path

from kaf_pas.ckk.views import files_askon

urlpatterns = [

    path('Files_askon/Fetch/', files_askon.Files_askon_Fetch),
    path('Files_askon/Add', files_askon.Files_askon_Add),
    path('Files_askon/Update', files_askon.Files_askon_Update),
    path('Files_askon/Remove', files_askon.Files_askon_Remove),
    path('Files_askon/Lookup/', files_askon.Files_askon_Lookup),
    path('Files_askon/Info/', files_askon.Files_askon_Info),
    path('Files_askon/Copy', files_askon.Files_askon_Copy),

]



import logging

from isc_common.models.audit import AuditManager, AuditQuerySet
from isc_common.models.base_ref import BaseRef

logger = logging.getLogger(__name__)


class StatusQuerySet(AuditQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class StatusManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            "id": record.id,
            "code": record.code,
            "name": record.name,
            "description": record.description,
            "lastmodified": record.lastmodified,
            "editing": record.editing,
            "deliting": record.deliting,
        }
        return res

    def get_queryset(self):
        return StatusQuerySet(self.model, using=self._db)


class Status(BaseRef):
    objects = StatusManager()

    def get_status(self, code):
        try:
            return self.objects.get(code=code)
        except Status.DoesNotExist:
            return None

    def __str__(self):
        return f"{self.id}"

    class Meta:
        verbose_name = 'Статусы'



# Generated by Django 3.0.4 on 2020-03-18 13:37

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0142_pathes_props'),
    ]

    operations = [
        migrations.AlterField(
            model_name='pathes',
            name='props',
            field=bitfield.models.BitField((('enable_upload', 'Директория для закачки'),), db_index=True, default=1),
        ),
    ]



# Generated by Django 3.0.3 on 2020-03-01 14:24

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0122_auto_20200301_0910'),
    ]

    operations = [
        migrations.AddField(
            model_name='operations',
            name='prev_status',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='planing_Operations_prev_status', to='planing.Status_operation_types'),
        ),
        migrations.AlterField(
            model_name='operations',
            name='status',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='planing_Operations_status', to='planing.Status_operation_types'),
        ),
    ]



from django.urls import path

from kaf_pas.sales.views import demand_materials

urlpatterns = [

    path('Demand_materials/Fetch/', demand_materials.Demand_materials_Fetch),
    path('Demand_materials/Add', demand_materials.Demand_materials_Add),
    path('Demand_materials/Update', demand_materials.Demand_materials_Update),
    path('Demand_materials/Remove', demand_materials.Demand_materials_Remove),
    path('Demand_materials/Lookup/', demand_materials.Demand_materials_Lookup),
    path('Demand_materials/Info/', demand_materials.Demand_materials_Info),
    path('Demand_materials/Copy', demand_materials.Demand_materials_Copy),

]



# Generated by Django 3.0.4 on 2020-03-26 19:22

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0143_auto_20200318_1337'),
    ]

    operations = [
        migrations.AddField(
            model_name='uploades',
            name='props',
            field=bitfield.models.BitField((('confirmed', 'Подтверждено'),), db_index=True, default=1),
        ),
    ]



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.ckk.models.locations import Locations, LocationsManager


@JsonResponseWithException()
def Locations_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Locations.objects.
                filter().
                get_range_rows1(
                request=request,
                function=LocationsManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Locations_Add(request):
    return JsonResponse(DSResponseAdd(data=Locations.objects.createFromRequest(request=request, propsArr=['isWorkshop']), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Locations_Update(request):
    return JsonResponse(DSResponseUpdate(data=Locations.objects.updateFromRequest(request=request, removed=['calendar__full_name'], propsArr=['isWorkshop']), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Locations_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Locations.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Locations_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Locations.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Locations_Info(request):
    return JsonResponse(DSResponse(request=request, data=Locations.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 2.2.4 on 2019-08-20 10:59

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0144_auto_20190820_1057'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='material_askon',
            unique_together={('field0', 'field1', 'field2', 'field3', 'field4', 'field5', 'field6', 'field7')},
        ),
    ]



# Generated by Django 2.2.7 on 2019-11-20 15:40

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0130_auto_20191120_1523'),
    ]

    operations = [
        migrations.AlterField(
            model_name='launch_item_refs',
            name='props',
            field=bitfield.models.BitField((('enabled', 'Доступность в данной производственной спецификации'), ('added', 'Добавлено в данную производственную спецификацию'), ('edited', 'Добавлено в данную производственную спецификацию')), db_index=True, default=1),
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-17 22:31

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0117_auto_20191217_0709'),
    ]

    operations = [
        migrations.AlterField(
            model_name='lotsman_documents_hierarcy',
            name='document',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='kd.Documents'),
        ),
        migrations.AlterField(
            model_name='lotsman_documents_hierarcy_refs',
            name='child',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='Lotsman_documents_hierarcy_child', to='kd.Lotsman_documents_hierarcy'),
        ),
        migrations.AlterField(
            model_name='lotsman_documents_hierarcy_refs',
            name='parent',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='Lotsman_documents_hierarcy_parent', to='kd.Lotsman_documents_hierarcy'),
        ),
    ]



# Generated by Django 2.2.2 on 2019-06-25 12:20

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0049_auto_20190624_1358'),
    ]

    operations = [
        migrations.AlterModelTable(
            name='pathes_view',
            table='kd_pathes_view',
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-07 18:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0158_auto_20191207_1031'),
    ]

    operations = [
        migrations.AlterField(
            model_name='launch_operations_item',
            name='num',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='launch_operations_item',
            name='qty',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
    ]



# Generated by Django 2.2.7 on 2019-11-19 10:32

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0115_auto_20191119_1031'),
    ]

    operations = [
        migrations.AlterField(
            model_name='launch_operations_item',
            name='props',
            field=bitfield.models.BitField((('enabled', 'Доступность в данной производственной спецификации'), ('added', 'Добавлено в данную производственную спецификацию'), ('edited', 'Изменено в данной производственной спецификации'), ('refreshed', 'Просмотрено при переформировании')), db_index=True, default=0),
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-30 17:47

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0151_auto_20190830_1745'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='status',
            name='code_owner',
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-28 17:19

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0148_auto_20190826_1127'),
        ('production', '0060_auto_20190822_2250'),
    ]

    operations = [
        migrations.AddField(
            model_name='operation_resources',
            name='location',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='ckk.Locations'),
        ),
        migrations.AlterField(
            model_name='operation_resources',
            name='resource',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='production.Resource'),
        ),
    ]



# Generated by Django 2.2 on 2019-04-25 18:04

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0003_operation_plan_template_locations_distribution_coefficient'),
    ]

    operations = [
        migrations.RenameField(
            model_name='operation_plan_template_locations',
            old_name='location',
            new_name='locations',
        ),
    ]



from django.urls import path

from kaf_pas.sales.views import status_precent

urlpatterns = [

    path('StatusPrecent/Fetch/', status_precent.StatusPrecent_Fetch),
    path('StatusPrecent/Add', status_precent.StatusPrecent_Add),
    path('StatusPrecent/Update', status_precent.StatusPrecent_Update),
    path('StatusPrecent/Remove', status_precent.StatusPrecent_Remove),
    path('StatusPrecent/Lookup/', status_precent.StatusPrecent_Lookup),
    path('StatusPrecent/Info/', status_precent.StatusPrecent_Info),
    path('StatusPrecent/Copy', status_precent.StatusPrecent_Copy),

]



# Generated by Django 2.2.1 on 2019-05-24 09:26

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0031_remove_demand_customer'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='demand',
            name='deliverydate',
        ),
        migrations.RemoveField(
            model_name='demand',
            name='maxlateness',
        ),
        migrations.RemoveField(
            model_name='demand',
            name='minshipment',
        ),
        migrations.RemoveField(
            model_name='demand',
            name='priority',
        ),
        migrations.RemoveField(
            model_name='demand',
            name='quantity',
        ),
    ]



# Generated by Django 2.2.2 on 2019-06-12 23:01

import bitfield.models
from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0095_auto_20190612_0517'),
    ]

    operations = [
        migrations.AlterField(
            model_name='item',
            name='props',
            field=bitfield.models.BitField((('relevant', 'Актуальность'), ('from_cdw', 'Получено из чертежа'), ('from_spw', 'Получено из спецификации'), ('for_line', 'Строка спецификации'), ('from_pdf', 'Получено из бумажного архива'), ('man_input', 'Занесено вручную')), db_index=True, default=1),
        ),
        migrations.AlterField(
            model_name='item_line',
            name='child',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='item_child', to='ckk.Item', verbose_name='Товарная позиция'),
        ),
        migrations.AlterField(
            model_name='item_line',
            name='parent',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='item_parent', to='ckk.Item', verbose_name='Товарная позиция'),
        ),
        migrations.AlterField(
            model_name='item_production_specification',
            name='item',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='ckk.Item'),
        ),
        migrations.AlterField(
            model_name='item_refs',
            name='child',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='child', to='ckk.Item'),
        ),
        migrations.AlterField(
            model_name='item_refs',
            name='parent',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='parent', to='ckk.Item'),
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-17 15:52

import django.contrib.postgres.fields.jsonb
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0193_auto_20200217_1355'),
    ]

    operations = [
        migrations.AddField(
            model_name='launches',
            name='facets',
            field=django.contrib.postgres.fields.jsonb.JSONField(default=dict),
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-30 17:44

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0148_auto_20190826_1127'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='status',
            name='code_owner',
        ),
    ]



import logging

from django.conf import settings
from django.db import transaction
from django.db.models import BigIntegerField, BooleanField, PositiveIntegerField
from django.forms import model_to_dict

from isc_common import delAttr, setAttr
from isc_common.fields.code_field import CodeField
from isc_common.fields.name_field import NameField
from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.models.base_ref import Hierarcy
from isc_common.models.tree_audit import TreeAuditModelManager, TreeAuditModelQuerySet
from isc_common.number import DelProps
from kaf_pas.ckk.models.item import Item_add, Item
from kaf_pas.ckk.models.item_line import Item_lineManager
from kaf_pas.ckk.models.item_refs import Item_refsManager
from kaf_pas.kd.models.document_attributes import Document_attributes
from kaf_pas.kd.models.documents import Documents
from kaf_pas.kd.models.lotsman_documents_hierarcy import Lotsman_documents_hierarcy

logger = logging.getLogger(__name__)


class Item_viewQuerySet(TreeAuditModelQuerySet):
    def get_range_rows(self, start=None, end=None, function=None, json=None, distinct_field_names=None, criteria=None, *args):
        queryResult = self._get_range_rows(*args, start=start, end=end, function=function, json=json, distinct_field_names=distinct_field_names)
        logger.debug(f'\n\n{queryResult.query}\n')

        is_filtered = False
        for criterion in criteria:
            if criterion.get('_constructor') == 'AdvancedCriteria':
                for criterion1 in criterion.get('criteria'):
                    if criterion1.get('fieldName') in ['STMP_2__value_str', 'STMP_1__value_str']:
                        is_filtered = True
                        break
                if is_filtered == True:
                    break

        if function:
            res = [function(record) for record in queryResult]
        else:
            res = [model_to_dict(record) for record in queryResult]

        return res

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Item_viewManager(TreeAuditModelManager):

    def updateFromRequest(self, request, removed=None, function=None):
        from kaf_pas.ckk.models.item_refs import Item_refs

        if not isinstance(request, DSRequest):
            request = DSRequest(request=request)
        data = request.get_data()

        with transaction.atomic():

            relevant = data.get('relevant')
            confirmed = data.get('confirmed')

            props = data.get('props', 0)
            refs_props = data.get('refs_props', 0)
            refs_id = data.get('refs_id')

            if relevant == 'Актуален':
                props |= Item.props.relevant
            else:
                props &= ~Item.props.relevant

            if confirmed == 'Подтвержден':
                props |= Item.props.confirmed
            else:
                props &= ~Item.props.confirmed

            where_from = data.get('where_from')
            if where_from == 'Получено из чертежа':
                props |= Item.props.from_cdw
            elif where_from == 'Получено из спецификации':
                props |= Item.props.from_spw
            elif where_from == 'Получено из бумажного архива':
                props |= Item.props.from_pdf
            elif where_from == 'Занесено вручную':
                props |= Item.props.man_input

            delAttr(data, 'relevant')
            delAttr(data, 'confirmed')
            delAttr(data, 'where_from')
            delAttr(data, 'qty_operations')
            delAttr(data, 'refs_props')
            delAttr(data, 'icon')
            setAttr(data, 'props', props)

            Item_refs.objects.filter(id=refs_id).update(props=refs_props)
            res = Item.objects.filter(id=data.get('id')).update(**data)

            return res

    @staticmethod
    def getRecord(record):
        res = {
            "id": record.id,
            "STMP_1_id": record.STMP_1.id if record.STMP_1 else None,
            "STMP_1__value_str": record.STMP_1.value_str if record.STMP_1 else None,
            "STMP_2_id": record.STMP_2.id if record.STMP_2 else None,
            "STMP_2__value_str": record.STMP_2.value_str if record.STMP_2 else None,
            "lastmodified": record.lastmodified,
            "document_id": record.document.id if record.document else None,
            "lotsman_document_id": record.lotsman_document.id if record.lotsman_document else None,
            "document__file_document": record.document.file_document if record.document else None,
            "parent_id": record.parent_id,
            "editing": record.editing,
            "deliting": record.deliting,
            "isFolder": record.isFolder,
            "relevant": record.relevant,
            "confirmed": record.confirmed,
            "section": record.section,
            "version": record.version,
            "where_from": record.where_from,
            "isLotsman": record.isLotsman,
            "props": record.props,
            "refs_props": record.refs_props,
            'icon': Item_lineManager.getIcon(record)
        }
        # print(res)
        return DelProps(res)

    def get_queryset(self):
        return Item_viewQuerySet(self.model, using=self._db)

    @staticmethod
    def copy_item(item, parent_item):
        from kaf_pas.ckk.models.item_refs import Item_refs
        from kaf_pas.ckk.models.item_line import Item_line
        from kaf_pas.ckk.models.item_image_refs import Item_image_refs

        if not isinstance(item, Item_view) and not isinstance(item, Item):
            raise Exception(f'item must be a Item type')

        settings.LOCKS.acquire('Item_viewManager.copy_item')
        dict_item = model_to_dict(item)
        delAttr(dict_item, 'id')
        delAttr(dict_item, 'relevant')
        delAttr(dict_item, 'where_from')
        delAttr(dict_item, 'isFolder')
        setAttr(dict_item, 'STMP_1_id', dict_item.get('STMP_1'))
        setAttr(dict_item, 'STMP_2_id', dict_item.get('STMP_2'))
        setAttr(dict_item, 'document_id', dict_item.get('document'))
        setAttr(dict_item, 'props', Item.props.relevant | Item.props.copied)
        new_item = Item.objects.create(**dict_item)

        Item_refs.objects.get_or_create(parent=parent_item, child=new_item)
        for item_refs in Item_refs.objects.filter(parent_id=item.id):
            Item_refs.objects.get_or_create(parent_id=new_item.id, child=item_refs.child)

        for item_line in Item_line.objects.filter(parent_id=item.id):
            dict_item_line = model_to_dict(item_line)

            delAttr(dict_item_line, 'id')
            setAttr(dict_item_line, 'parent_id', new_item.id)
            delAttr(dict_item_line, 'parent')
            setAttr(dict_item_line, 'child_id', dict_item_line.get('child'))
            delAttr(dict_item_line, 'child')

            setAttr(dict_item_line, 'SPC_CLM_FORMAT_id', dict_item_line.get('SPC_CLM_FORMAT'))
            delAttr(dict_item_line, 'SPC_CLM_FORMAT')
            setAttr(dict_item_line, 'SPC_CLM_ZONE_id', dict_item_line.get('SPC_CLM_ZONE'))
            delAttr(dict_item_line, 'SPC_CLM_ZONE')
            setAttr(dict_item_line, 'SPC_CLM_POS_id', dict_item_line.get('SPC_CLM_POS'))
            delAttr(dict_item_line, 'SPC_CLM_POS')
            setAttr(dict_item_line, 'SPC_CLM_MARK_id', dict_item_line.get('SPC_CLM_MARK'))
            delAttr(dict_item_line, 'SPC_CLM_MARK')
            setAttr(dict_item_line, 'SPC_CLM_NAME_id', dict_item_line.get('SPC_CLM_NAME'))
            delAttr(dict_item_line, 'SPC_CLM_NAME')
            setAttr(dict_item_line, 'SPC_CLM_COUNT_id', dict_item_line.get('SPC_CLM_COUNT'))
            delAttr(dict_item_line, 'SPC_CLM_COUNT')
            setAttr(dict_item_line, 'SPC_CLM_NOTE_id', dict_item_line.get('SPC_CLM_NOTE'))
            delAttr(dict_item_line, 'SPC_CLM_NOTE')
            setAttr(dict_item_line, 'SPC_CLM_MASSA_id', dict_item_line.get('SPC_CLM_MASSA'))
            delAttr(dict_item_line, 'SPC_CLM_MASSA')
            setAttr(dict_item_line, 'SPC_CLM_MATERIAL_id', dict_item_line.get('SPC_CLM_MATERIAL'))
            delAttr(dict_item_line, 'SPC_CLM_MATERIAL')
            setAttr(dict_item_line, 'SPC_CLM_USER_id', dict_item_line.get('SPC_CLM_USER'))
            delAttr(dict_item_line, 'SPC_CLM_USER')
            setAttr(dict_item_line, 'SPC_CLM_KOD_id', dict_item_line.get('SPC_CLM_KOD'))
            delAttr(dict_item_line, 'SPC_CLM_KOD')
            setAttr(dict_item_line, 'SPC_CLM_FACTORY_id', dict_item_line.get('SPC_CLM_FACTORY'))
            delAttr(dict_item_line, 'SPC_CLM_FACTORY')
            Item_line.objects.get_or_create(**dict_item_line)

        for item_image_refs in Item_image_refs.objects.filter(item_id=item.id):
            dict_item_image_refs = model_to_dict(item_image_refs)
            delAttr(dict_item_image_refs, 'id')
            delAttr(dict_item_image_refs, 'item')
            setAttr(dict_item_image_refs, 'item_id', new_item.id)
            setAttr(dict_item_image_refs, 'thumb_id', dict_item_image_refs.get('thumb'))
            delAttr(dict_item_image_refs, 'thumb')
            setAttr(dict_item_image_refs, 'thumb10_id', dict_item_image_refs.get('thumb10'))
            delAttr(dict_item_image_refs, 'thumb10')
            Item_image_refs.objects.get_or_create(**dict_item_image_refs)
        settings.LOCKS.release('Item_viewManager.copy_item')

        return new_item

    def copyFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()

        with transaction.atomic():
            item = self.get(id=data.get('id'), parent_id=data.get('parent_id'))
            parent_item = Item.objects.get(id=data.get('parent_id'))

            new_item = Item_viewManager.copy_item(item=item, parent_item=parent_item)
            res = model_to_dict(new_item)
            setAttr(res, 'props', res.get('props')._value)

        return res


class Item_view(Hierarcy):
    STMP_1 = ForeignKeyProtect(Document_attributes, verbose_name='Наименование изделия', related_name='STMP_1_view', null=True, blank=True)
    STMP_2 = ForeignKeyProtect(Document_attributes, verbose_name='Обозначение изделия', related_name='STMP_2_view', null=True, blank=True)
    document = ForeignKeyProtect(Documents, verbose_name='Документ', null=True, blank=True)
    lotsman_document = ForeignKeyProtect(Lotsman_documents_hierarcy, verbose_name='Документ из Лоцмана', null=True, blank=True)
    refs_id = BigIntegerField()
    isLotsman = BooleanField()
    relevant = NameField()
    confirmed = NameField()
    where_from = NameField()
    props = Item_add.get_prop_field()
    refs_props = Item_refsManager.props()
    version = PositiveIntegerField(null=True, blank=True)
    section = CodeField(null=True, blank=True)

    isFolder = BooleanField()

    objects = Item_viewManager()

    @property
    def item(self):
        from kaf_pas.ckk.models.item import Item
        return Item.objects.get(id=self.id)

    def __str__(self):
        return f"ID={self.id} STMP_1=[{self.STMP_1}], STMP_2=[{self.STMP_2}], props={self.props}"

    class Meta:
        managed = False
        db_table = 'ckk_item_view'
        verbose_name = 'Товарная позиция'



# Generated by Django 2.2.8 on 2019-12-18 16:42

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0120_auto_20191218_1014'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='lotsman_document_attr_cross',
            unique_together=set(),
        ),
    ]



# Generated by Django 3.0.3 on 2020-03-01 09:10

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0121_auto_20200301_0511'),
    ]

    operations = [
        migrations.AlterField(
            model_name='status_operation_types',
            name='opertype',
            field=isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='planing.Operation_types'),
        ),
        migrations.AlterModelTable(
            name='production_order',
            table=None,
        ),
    ]



from django.urls import path

from kaf_pas.input.views import user_agg_view

urlpatterns = [

    path('User_agg_view/Fetch/', user_agg_view.User_agg_view_Fetch),
    path('User_agg_view/Add', user_agg_view.User_agg_view_Add),
    path('User_agg_view/Update', user_agg_view.User_agg_view_Update),
    path('User_agg_view/Remove', user_agg_view.User_agg_view_Remove),
    path('User_agg_view/Lookup/', user_agg_view.User_agg_view_Lookup),
    path('User_agg_view/Info/', user_agg_view.User_agg_view_Info),
    path('User_agg_view/Copy', user_agg_view.User_agg_view_Copy),

]



from one_c.models.entities import EntityProperties, Field


class Nomenklatura(EntityProperties):
    code = 'Номенклатура'

    def used_field(self):
        return [
            Field(name='Ref', primary_key=True, title='ID', hidden=True, type=Field.uuid()),
            Field(name='Code', title='Код номенклатуры (1C)', lookup=True, canSort=True),
            Field(name='full_name', title='Полное наименование номенклатуры (1C)', lookup=True, canSort=False, canEdit=False, canFilter=False, only_datasource=True),
            Field(name='Description', title='Наименование номенклатуры (1C)', lookup=True, canSort=True),
            Field(name='Parent', hidden=True, type=Field.uuid()),
            Field(name='IsFolder', hidden=True, type=Field.boolean()),
            Field(name='НаименованиеПолное', title='Полное наименование номенклатуры  (1C)', lookup=True, canSort=True),
            Field(name='ВидНоменклатуры', hidden=True, type=Field.uuid()),
            Field(name='ЕдиницаДляОтчетов', hidden=True, type=Field.uuid()),
        ]



# Generated by Django 3.0.3 on 2020-02-08 07:14

from django.db import migrations
import django_db_constraints.operations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0192_auto_20200206_1540'),
    ]

    operations = [
        django_db_constraints.operations.AlterConstraints(
            name='item',
            db_constraints={'Item_not_null_STMP1_STMP2': 'CHECK ("STMP_1_id" IS NOT NULL OR "STMP_2_id" IS NOT NULL)'},
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-22 07:21

import django.db.models.deletion
from django.db import migrations

import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0158_delete_status'),
        ('production', '0074_item_refs'),
    ]

    operations = [
        migrations.AddField(
            model_name='operations_item',
            name='prod_item',
            field=isc_common.fields.related.ForeignKeySetNull(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='op_prod_item', to='ckk.Item'),
        ),
        migrations.AlterField(
            model_name='operations_item',
            name='item',
            field=isc_common.fields.related.ForeignKeySetNull(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='op_item', to='ckk.Item'),
        ),
        migrations.AlterUniqueTogether(
            name='item_refs',
            unique_together={('item', 'child', 'parent')},
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-06 14:47

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0127_auto_20200131_1121'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='documents_thumb',
            unique_together={('document', 'lotsman_document', 'path')},
        ),
        migrations.AlterUniqueTogether(
            name='documents_thumb10',
            unique_together={('document', 'lotsman_document', 'path')},
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-09 11:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0174_auto_20200209_1126'),
    ]

    operations = [
        migrations.AddConstraint(
            model_name='launch_operation_resources',
            constraint=models.UniqueConstraint(condition=models.Q(('location', None), ('resource', None)), fields=('launch_operationitem',), name='Launch_operation_resources_unique_constraint_0'),
        ),
        migrations.AddConstraint(
            model_name='launch_operation_resources',
            constraint=models.UniqueConstraint(condition=models.Q(location=None), fields=('launch_operationitem', 'resource'), name='Launch_operation_resources_unique_constraint_1'),
        ),
        migrations.AddConstraint(
            model_name='launch_operation_resources',
            constraint=models.UniqueConstraint(condition=models.Q(resource=None), fields=('launch_operationitem', 'location'), name='Launch_operation_resources_unique_constraint_2'),
        ),
        migrations.AddConstraint(
            model_name='launch_operation_resources',
            constraint=models.UniqueConstraint(fields=('launch_operationitem', 'location', 'resource'), name='Launch_operation_resources_unique_constraint_3'),
        ),
        migrations.AddConstraint(
            model_name='launch_operations_material',
            constraint=models.UniqueConstraint(condition=models.Q(('material', None), ('material_askon', None)), fields=('launch_operationitem',), name='Launch_operations_material_unique_constraint_0'),
        ),
        migrations.AddConstraint(
            model_name='launch_operations_material',
            constraint=models.UniqueConstraint(condition=models.Q(material_askon=None), fields=('launch_operationitem', 'material'), name='Launch_operations_material_unique_constraint_1'),
        ),
        migrations.AddConstraint(
            model_name='launch_operations_material',
            constraint=models.UniqueConstraint(condition=models.Q(material=None), fields=('launch_operationitem', 'material_askon'), name='Launch_operations_material_unique_constraint_2'),
        ),
        migrations.AddConstraint(
            model_name='launch_operations_material',
            constraint=models.UniqueConstraint(fields=('launch_operationitem', 'material', 'material_askon'), name='Launch_operations_material_unique_constraint_3'),
        ),
        migrations.AddConstraint(
            model_name='operation_def_resources',
            constraint=models.UniqueConstraint(condition=models.Q(('location', None), ('resource', None)), fields=('operation',), name='Operation_def_resources_unique_constraint_0'),
        ),
        migrations.AddConstraint(
            model_name='operation_def_resources',
            constraint=models.UniqueConstraint(condition=models.Q(location=None), fields=('operation', 'resource'), name='Operation_def_resources_unique_constraint_1'),
        ),
        migrations.AddConstraint(
            model_name='operation_def_resources',
            constraint=models.UniqueConstraint(condition=models.Q(resource=None), fields=('location', 'operation'), name='Operation_def_resources_unique_constraint_2'),
        ),
        migrations.AddConstraint(
            model_name='operation_def_resources',
            constraint=models.UniqueConstraint(fields=('location', 'operation', 'resource'), name='Operation_def_resources_unique_constraint_3'),
        ),
        migrations.AddConstraint(
            model_name='operation_material',
            constraint=models.UniqueConstraint(condition=models.Q(('material', None), ('material_askon', None)), fields=('operationitem',), name='Operation_material_unique_constraint_0'),
        ),
        migrations.AddConstraint(
            model_name='operation_material',
            constraint=models.UniqueConstraint(condition=models.Q(material_askon=None), fields=('material', 'operationitem'), name='Operation_material_unique_constraint_1'),
        ),
        migrations.AddConstraint(
            model_name='operation_material',
            constraint=models.UniqueConstraint(condition=models.Q(material=None), fields=('material_askon', 'operationitem'), name='Operation_material_unique_constraint_2'),
        ),
        migrations.AddConstraint(
            model_name='operation_material',
            constraint=models.UniqueConstraint(fields=('material', 'material_askon', 'operationitem'), name='Operation_material_unique_constraint_3'),
        ),
        migrations.AddConstraint(
            model_name='operation_resources',
            constraint=models.UniqueConstraint(condition=models.Q(('location', None), ('resource', None)), fields=('operationitem',), name='Operation_resources_unique_constraint_0'),
        ),
        migrations.AddConstraint(
            model_name='operation_resources',
            constraint=models.UniqueConstraint(condition=models.Q(location=None), fields=('operationitem', 'resource'), name='Operation_resources_unique_constraint_1'),
        ),
        migrations.AddConstraint(
            model_name='operation_resources',
            constraint=models.UniqueConstraint(condition=models.Q(resource=None), fields=('location', 'operationitem'), name='Operation_resources_unique_constraint_2'),
        ),
        migrations.AddConstraint(
            model_name='operation_resources',
            constraint=models.UniqueConstraint(fields=('location', 'operationitem', 'resource'), name='Operation_resources_unique_constraint_3'),
        ),
    ]



# Generated by Django 3.0.3 on 2020-03-02 04:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0226_launch_item_refs_qty_per_one'),
    ]

    operations = [
        migrations.AlterField(
            model_name='launch_item_refs',
            name='qty_per_one',
            field=models.FloatField(blank=True, null=True),
        ),
    ]



from django.urls import path

from kaf_pas.ckk.views import material_added_values

urlpatterns = [

    path('Material_added_values/Fetch/', material_added_values.Material_added_values_Fetch),
    path('Material_added_values/Add', material_added_values.Material_added_values_Add),
    path('Material_added_values/Update', material_added_values.Material_added_values_Update),
    path('Material_added_values/Remove', material_added_values.Material_added_values_Remove),
    path('Material_added_values/Lookup/', material_added_values.Material_added_values_Lookup),
    path('Material_added_values/Info/', material_added_values.Material_added_values_Info),
    path('Material_added_values/Copy', material_added_values.Material_added_values_Copy),

]



# Generated by Django 2.2.7 on 2019-11-19 14:03

from django.db import migrations
import isc_common.fields.description_field


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0118_launch_item_refs_added_launch'),
    ]

    operations = [
        migrations.AddField(
            model_name='launch_item_refs',
            name='description',
            field=isc_common.fields.description_field.DescriptionField(),
        ),
    ]



import logging

from bitfield import BitField
from django.db.models import TextField, CharField

from isc_common.fields.code_field import CodeField
from isc_common.models.audit import AuditQuerySet, AuditManager
from isc_common.models.base_ref import Hierarcy

logger = logging.getLogger(__name__)


class CustomerQuerySet(AuditQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class CustomerManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            "id": record.id,
            "inn": record.inn,
            "kpp": record.kpp,
            "name": record.name,
            "long_name": record.long_name,
            "full_name": record.full_name,
            "description": record.description,
            "parent_id": record.parent.id if record.parent else None,
            "lastmodified": record.lastmodified,
            "editing": record.editing,
            "deliting": record.deliting,
        }
        return res

    def get_queryset(self):
        return CustomerQuerySet(self.model, using=self._db)


class Customer(Hierarcy):
    inn = CodeField(verbose_name="ИНН", max_length=12, db_index=True, null=True, blank=True)
    kpp = CodeField(verbose_name="КПП", max_length=9, db_index=True, null=True, blank=True)
    name = CharField(verbose_name="Наименование", max_length=255, db_index=True)
    long_name = TextField(verbose_name="Полное наименование", null=True, blank=True)
    description = TextField(verbose_name="Описание", null=True, blank=True)
    props = BitField(flags=(
        ('real', 'Реальный клиент'),
    ), default=1, db_index=True)

    objects = CustomerManager()

    def __str__(self):
        return f"{self.id}"

    class Meta:
        verbose_name = 'Клиент'



# Generated by Django 2.2.6 on 2019-10-14 01:21

import bitfield.models
from django.db import migrations, models
import django.utils.timezone
import isc_common.fields.name_field


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0154_auto_20191010_2032'),
    ]

    operations = [
        migrations.CreateModel(
            name='Item_operations_view',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('parent_id', models.BigIntegerField()),
                ('relevant', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('where_from', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('props', bitfield.models.BitField((('relevant', 'Актуальность'), ('from_cdw', 'Получено из чертежа'), ('from_spw', 'Получено из спецификации'), ('for_line', 'Строка спецификации'), ('from_pdf', 'Получено из бумажного архива'), ('man_input', 'Занесено вручную'), ('copied', 'Скопировано')), db_index=True, default=1)),
                ('version', models.PositiveIntegerField(blank=True, null=True)),
                ('qty_operations', models.PositiveIntegerField()),
                ('isFolder', models.BooleanField()),
            ],
            options={
                'verbose_name': 'Товарная позиция',
                'db_table': 'ckk_item_view',
                'managed': False,
            },
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-04 14:24

from django.db import migrations
import isc_common.fields.name_field


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0137_auto_20191204_1424'),
    ]

    operations = [
        migrations.AlterField(
            model_name='operations',
            name='name',
            field=isc_common.fields.name_field.NameStrictField(),
        ),
    ]



import logging

from bitfield import BitField
from django.db.models import PositiveIntegerField

from isc_common.bit import IsBitOn
from isc_common.fields.code_field import CodeStrictField
from isc_common.fields.description_field import DescriptionField
from isc_common.fields.name_field import NameField
from isc_common.fields.related import ForeignKeyProtect, ForeignKeySetNull
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
from isc_common.models.tree_audit import TreeAuditModelManager
from kaf_pas.ckk.models.item import Item
from kaf_pas.ckk.models.item_line import Item_line
from kaf_pas.kd.models.document_attributes import Document_attributes
from kaf_pas.production.models.launches import Launches

logger = logging.getLogger(__name__)


class Launch_item_lineQuerySet(AuditQuerySet):

    def create(self, **kwargs):
        return super().create(**kwargs)

    def get_or_create(self, defaults=None, **kwargs):
        return super().get_or_create(defaults, **kwargs)

    def update(self, **kwargs):
        return super().update(**kwargs)

    def update_or_create(self, defaults=None, **kwargs):
        return super().get_or_create(defaults, **kwargs)


class Launch_item_lineManager(TreeAuditModelManager):
    @staticmethod
    def props():
        return  BitField(flags=(
        ('enabled', 'Доступность в данной производственной спецификации'),  # 1
    ), default=1, db_index=True)

    def get_queryset(self):
        return Launch_item_lineQuerySet(self.model, using=self._db)

    @staticmethod
    def section_num(section):
        sorted_section = {"Документация": 0, "Комплексы": 1, "Сборочные единицы": 2, "Детали": 3, "Стандартные изделия": 4, "Прочие изделия": 5, "Материалы": 6, "Комплекты": 7}
        return sorted_section.get(section)

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'child_id': record.child.id if record.child else None,
            'parent_id': record.parent.id,
            'SPC_CLM_FORMAT_id': record.SPC_CLM_FORMAT.id if record.SPC_CLM_FORMAT else None,
            'SPC_CLM_FORMAT__value_str': record.SPC_CLM_FORMAT.value_str if record.SPC_CLM_FORMAT else None,
            'SPC_CLM_ZONE_id': record.SPC_CLM_ZONE.id if record.SPC_CLM_ZONE else None,
            'SPC_CLM_ZONE__value_str': record.SPC_CLM_ZONE.value_str if record.SPC_CLM_ZONE else None,
            'SPC_CLM_POS_id': record.SPC_CLM_POS.id if record.SPC_CLM_POS else None,
            'SPC_CLM_POS__value_str': record.SPC_CLM_POS.value_str if record.SPC_CLM_POS else None,
            'SPC_CLM_POS__value_int': record.SPC_CLM_POS.value_int if record.SPC_CLM_POS else None,
            'SPC_CLM_MARK_id': record.SPC_CLM_MARK.id if record.SPC_CLM_MARK else None,
            'SPC_CLM_MARK__value_str': record.SPC_CLM_MARK.value_str if record.SPC_CLM_MARK else None,
            'SPC_CLM_NAME_id': record.SPC_CLM_NAME.id if record.SPC_CLM_NAME else None,
            'SPC_CLM_NAME__value_str': record.SPC_CLM_NAME.value_str if record.SPC_CLM_NAME else None,
            'SPC_CLM_COUNT_id': record.SPC_CLM_COUNT.id if record.SPC_CLM_COUNT else None,
            'SPC_CLM_COUNT__value_str': record.SPC_CLM_COUNT.value_str if record.SPC_CLM_COUNT else None,
            'SPC_CLM_COUNT__value_int': record.SPC_CLM_COUNT.value_int if record.SPC_CLM_COUNT else None,
            'SPC_CLM_NOTE_id': record.SPC_CLM_NOTE.id if record.SPC_CLM_NOTE else None,
            'SPC_CLM_NOTE__value_str': record.SPC_CLM_NOTE.value_str if record.SPC_CLM_NOTE else None,
            'SPC_CLM_MASSA_id': record.SPC_CLM_MASSA.id if record.SPC_CLM_MASSA else None,
            'SPC_CLM_MASSA__value_str': record.SPC_CLM_MASSA.value_str if record.SPC_CLM_MASSA else None,
            'SPC_CLM_MATERIAL_id': record.SPC_CLM_MATERIAL.id if record.SPC_CLM_MATERIAL else None,
            'SPC_CLM_MATERIAL__value_str': record.SPC_CLM_MATERIAL.value_str if record.SPC_CLM_MATERIAL else None,
            'SPC_CLM_USER_id': record.SPC_CLM_USER.id if record.SPC_CLM_USER else None,
            'SPC_CLM_USER__value_str': record.SPC_CLM_USER.value_str if record.SPC_CLM_USER else None,
            'SPC_CLM_KOD_id': record.SPC_CLM_KOD.id if record.SPC_CLM_KOD else None,
            'SPC_CLM_KOD__value_str': record.SPC_CLM_KOD.value_str if record.SPC_CLM_KOD else None,
            'SPC_CLM_FACTORY_id': record.SPC_CLM_FACTORY.id if record.SPC_CLM_FACTORY else None,
            'SPC_CLM_FACTORY__value_str': record.SPC_CLM_FACTORY.value_str if record.SPC_CLM_FACTORY else None,
            "launch_id": record.launch.id,
            "item_line_id": record.item_line.id,
            'section': record.section,
            'section_num': record.section_num,
            'subsection': record.subsection,
            'lastmodified': record.lastmodified,
            'editing': record.editing,
            'deliting': record.deliting,
            'props': record.props._value,
            'enabled': IsBitOn(record.props, 0),
        }
        return res

    @staticmethod
    def getIcon(record):
        if record.isFolder:
            return 'folder_256.png'
        elif record.section:
            _section = record.section.lower()
            if _section == 'документация':
                return 'documentation.png'
            elif _section == 'комплексы':
                return 'complexes.png'
            elif _section == 'сборочные единицы':
                return 'assembly.png'
            elif _section == 'детали':
                return 'part.png'
            elif _section == 'стандартные изделия':
                return 'standard_prod.png'
            elif _section == 'прочие изделия':
                return 'other.png'
            elif _section == 'материалы':
                return 'materials.png'
            elif _section == 'комплекты':
                return 'kits.png'
            else:
                return 'folder_256.png'


class Launch_item_line(AuditModel):
    parent = ForeignKeyProtect(Item, related_name='Launch_item_parent')
    added_parent = ForeignKeyProtect('self', related_name='Launch_item_added_parent', null=True, blank=True)
    child = ForeignKeyProtect(Item, related_name='Launch_item_child')
    item_line = ForeignKeySetNull(Item_line, null=True, blank=True)
    launch = ForeignKeyProtect(Launches)

    SPC_CLM_FORMAT = ForeignKeyProtect(Document_attributes, verbose_name='Формат', related_name='Launch_SPC_CLM_FORMAT', null=True, blank=True)
    SPC_CLM_ZONE = ForeignKeyProtect(Document_attributes, verbose_name='Зона', related_name='Launch_SPC_CLM_ZONE', null=True, blank=True)
    SPC_CLM_POS = ForeignKeyProtect(Document_attributes, verbose_name='Позиция', related_name='Launch_SPC_CLM_POS', null=True, blank=True)
    SPC_CLM_MARK = ForeignKeyProtect(Document_attributes, verbose_name='Обозначение', related_name='Launch_SPC_CLM_MARK', null=True, blank=True)
    SPC_CLM_NAME = ForeignKeyProtect(Document_attributes, verbose_name='Наименование', related_name='Launch_SPC_CLM_NAME', null=True, blank=True)
    SPC_CLM_COUNT = ForeignKeyProtect(Document_attributes, verbose_name='Количество', related_name='Launch_SPC_CLM_COUNT', null=True, blank=True)
    SPC_CLM_NOTE = ForeignKeyProtect(Document_attributes, verbose_name='Примечание', related_name='Launch_SPC_CLM_NOTE', null=True, blank=True)
    SPC_CLM_MASSA = ForeignKeyProtect(Document_attributes, verbose_name='Масса', related_name='Launch_SPC_CLM_MASSA', null=True, blank=True)
    SPC_CLM_MATERIAL = ForeignKeyProtect(Document_attributes, verbose_name='Материал', related_name='Launch_SPC_CLM_MATERIAL', null=True, blank=True)
    SPC_CLM_USER = ForeignKeyProtect(Document_attributes, verbose_name='Пользовательская', related_name='Launch_SPC_CLM_USER', null=True, blank=True)
    SPC_CLM_KOD = ForeignKeyProtect(Document_attributes, verbose_name='Код', related_name='Launch_SPC_CLM_KOD', null=True, blank=True)
    SPC_CLM_FACTORY = ForeignKeyProtect(Document_attributes, verbose_name='Предприятие - изготовитель', related_name='Launch_SPC_CLM_FACTORY', null=True, blank=True)
    section = CodeStrictField()
    section_num = PositiveIntegerField(null=True, blank=True)
    subsection = NameField()
    description = DescriptionField()
    props = Launch_item_lineManager.props()


    objects = Launch_item_lineManager()

    @property
    def item_name(self):
        if self.SPC_CLM_NAME != None and self.SPC_CLM_MARK != None:
            return f'{self.SPC_CLM_NAME.value_str}: {self.SPC_CLM_MARK.value_str}'
        elif self.SPC_CLM_NAME == None and self.SPC_CLM_MARK != None:
            return self.SPC_CLM_MARK.value_str
        elif self.SPC_CLM_NAME != None and self.SPC_CLM_MARK == None:
            return self.SPC_CLM_NAME.value_str
        else:
            return 'Неизвестен'

    def save(self, *args, **kwargs):
        if self.parent:
            if self.parent.id == self.child.id:
                raise Exception(f'Attempt to record with parent.id ({self.parent.id}) == child.id ({self.child.id})')

        if self.SPC_CLM_MARK == None and self.SPC_CLM_NAME == None:
            raise Exception(f'SPC_CLM_MARK and SPC_CLM_NAME not been Null together. ({self})')
        super().save(*args, **kwargs)

    def __str__(self):
        return f'\n\nchild=[{self.child}]\n' \
               f'\nparent=[{self.parent}]\n' \
               f'\nSPC_CLM_FORMAT=[{self.SPC_CLM_FORMAT}]' \
               f'\nSPC_CLM_ZONE=[{self.SPC_CLM_ZONE}]' \
               f'\nSPC_CLM_POS=[{self.SPC_CLM_POS}]' \
               f'\nSPC_CLM_MARK=[{self.SPC_CLM_MARK}]' \
               f'\nSPC_CLM_NAME=[{self.SPC_CLM_NAME}]' \
               f'\nSPC_CLM_COUNT=[{self.SPC_CLM_COUNT}]' \
               f'\nSPC_CLM_NOTE=[{self.SPC_CLM_NOTE}]' \
               f'\nSPC_CLM_MASSA=[{self.SPC_CLM_MASSA}]' \
               f'\nSPC_CLM_MATERIAL=[{self.SPC_CLM_MATERIAL}]' \
               f'\nSPC_CLM_USER=[{self.SPC_CLM_USER}]' \
               f'\nSPC_CLM_KOD=[{self.SPC_CLM_KOD}' \
               f'\nSPC_CLM_FACTORY={self.SPC_CLM_FACTORY}]\n' \
               f'section={self.section}\n' \
               f'subsection={self.subsection}\n\n'

    class Meta:
        verbose_name = 'Строка состава изделия'
        unique_together = (('parent', 'child', 'launch'),)
        ordering = ['section']



from django.urls import path

from kaf_pas.sales.views import precent_dogovors

urlpatterns = [

    path('Precent_dogovors/Fetch/', precent_dogovors.Precent_dogovors_Fetch),
    path('Precent_dogovors/Add', precent_dogovors.Precent_dogovors_Add),
    path('Precent_dogovors/Update', precent_dogovors.Precent_dogovors_Update),
    path('Precent_dogovors/Remove', precent_dogovors.Precent_dogovors_Remove),
    path('Precent_dogovors/Lookup/', precent_dogovors.Precent_dogovors_Lookup),
    path('Precent_dogovors/Info/', precent_dogovors.Precent_dogovors_Info),
    path('Precent_dogovors/Copy', precent_dogovors.Precent_dogovors_Copy),

]



# Generated by Django 3.0.3 on 2020-02-23 05:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0200_resource_props'),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name='resource',
            name='Resource_unique_constraint_0',
        ),
        migrations.RemoveConstraint(
            model_name='resource',
            name='Resource_unique_constraint_1',
        ),
        migrations.AddConstraint(
            model_name='resource',
            constraint=models.UniqueConstraint(condition=models.Q(calendar=None), fields=('location', 'props'), name='Resource_unique_constraint_0'),
        ),
        migrations.AddConstraint(
            model_name='resource',
            constraint=models.UniqueConstraint(fields=('calendar', 'location', 'props'), name='Resource_unique_constraint_1'),
        ),
    ]



import logging

from django.db.models import DateTimeField, PositiveIntegerField, BooleanField

from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.models.base_ref import BaseRefHierarcy, BaseRefManager, BaseRefQuerySet
from isc_common.number import DelProps
from kaf_pas.production.models.status_launch import Status_launch
from kaf_pas.sales.models.demand import Demand

logger = logging.getLogger(__name__)


class Launches_viewQuerySet(BaseRefQuerySet):
    pass


class Launches_viewManager(BaseRefManager):
    # @staticmethod
    # def make_launch(data):
    #     from isc_common.auth.models.user import User
    #     from kaf_pas.ckk.models.item_refs import Item_refs
    #
    #     demand_id = getAttr(data, 'demand_id')
    #     qty = getAttr(data, 'qty')
    #     code = getAttr(data, 'code')
    #     date = getAttr(data, 'date')
    #     date = StrToDate(date, '%Y-%m-%d')
    #     user = User.objects.get(id=getAttr(data, 'user_id'))
    #     status, _ = Status_launch.objects.get_or_create(name='Формирование')
    #
    #     with transaction.atomic():
    #         for demand in Demand.objects.select_for_update().filter(id=demand_id):
    #             tail_qty = Demand_view.objects.get(id=demand_id).tail_qty
    #             if tail_qty < qty:
    #                 raise Exception(f'Затребованное количестово для запуска ({qty}), превышает возможное к запуску ({tail_qty})')
    #
    #             progress = ProgressStack(
    #                 host=settings.WS_HOST,
    #                 port=settings.WS_PORT,
    #                 channel=f'common_{user.username}',
    #             )
    #
    #             id = f'demand_{demand.id}'
    #             demand_str = f'<h3>Формирование запуска: Заказ № {demand.code} от {demand.date}</h3>'
    #
    #             progress.show(
    #                 title=f'<b>Обработано товарных позиций</b>',
    #                 label_contents=demand_str,
    #                 cntAll=Item_refs.objects.get_descendants_count(id=demand.precent_item.item.id),
    #                 id=id
    #             )
    #
    #             cnt = 0
    #             Item_refs.objects.filter(child=F('parent')).delete()
    #             launch = Launches_view.objects.create(
    #                 code=code,
    #                 demand_id=demand_id,
    #                 qty=qty,
    #                 date=date,
    #                 status=status
    #             )
    #             for item_ref in Item_refs.objects.get_descendants(id=demand.precent_item.item.id):
    #                 print(item_ref)
    #
    #     return data

    def createFromRequest(self, request):
        request = DSRequest(request=request)
        return Launches_viewManager.make_launch(request.get_data())

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'code': record.code,
            'name': record.name,
            'date': record.date,
            'description': record.description,
            'parent_id': record.parent.id if record.parent else None,

            'demand_id': record.demand.id,
            'demand__code': record.demand.code,
            'demand__date': record.demand.date,

            'status_id': record.status.id,
            'status__code': record.status.code,
            'status__name': record.status.name,

            'qty': record.qty,
            'isFolder': record.isFolder,
            # 'props': record.props,

            'editing': record.editing,
            'deliting': record.deliting,
        }
        return DelProps(res)

    def get_queryset(self):
        return Launches_viewQuerySet(self.model, using=self._db)


class Launches_view(BaseRefHierarcy):
    date = DateTimeField()
    status = ForeignKeyProtect(Status_launch)
    demand = ForeignKeyProtect(Demand)
    qty = PositiveIntegerField()
    isFolder = BooleanField()
    # props = LaunchesManager.props()

    objects = Launches_viewManager()

    def __str__(self):
        return f"ID:{self.id}, code: {self.code}, name: {self.name}, description: {self.description}"

    class Meta:
        verbose_name = 'Запуски'
        db_table = 'production_launches_view'
        managed = False



from django.conf import settings

from isc_common.http.DSRequest import DSRequest
from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.planing.models.operations_view import Operations_view, Operations_viewManager
from kaf_pas.planing.models.routing import Routing, RoutingManager


@JsonResponseWithException()
def Routing_Fetch(request):
    _request = DSRequest(request=request)
    data = _request.get_data()
    item_id = data.get('item_id')
    resource_id = data.get('resource_id')
    opers_type = settings.OPERS_TYPES_STACK.ROUTING_TASK.id

    return JsonResponse(
        DSResponse(
            request=request,
            data=Routing.objects.raw(params=[
                item_id, #
                resource_id, #
                item_id,  #
                opers_type, #
                item_id,  #
                resource_id, #
                opers_type,  #
                item_id,  #
                opers_type, #
                item_id,  #
                resource_id, #
                opers_type,  #
            ], function=RoutingManager.getRecord),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Operations_FetchView(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Operations_view.objects.
                filter().
                get_range_rows1(
                request=request,
                function=Operations_viewManager.getRecord,
                distinct_field_names=['id']
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Routing_Add(request):
    return JsonResponse(DSResponseAdd(data=Routing.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Routing_Update(request):
    return JsonResponse(DSResponseUpdate(data=Routing.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Routing_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Routing.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Routing_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Routing.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Routing_Info(request):
    return JsonResponse(DSResponse(request=request, data=Routing.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Routing_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Routing.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Routing_FetchLevels(request):
    return JsonResponse(DSResponse(request=request, data=Routing.objects.fetchLevelsFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Routing_FetchLocationsLevel(request):
    return JsonResponse(DSResponse(request=request, data=Routing.objects.fetchLocationsLevelFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Routing_FetchResourcesLevel(request):
    return JsonResponse(DSResponse(request=request, data=Routing.objects.fetchResourcesLevelFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 2.2.8 on 2019-12-08 09:24

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0049_auto_20191208_0924'),
    ]

    operations = [
        migrations.AlterField(
            model_name='operation_locations',
            name='location',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='planing_resource_loc', to='ckk.Locations'),
        ),
        migrations.AlterField(
            model_name='operation_locations',
            name='operation',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='planing_operation_loc', to='planing.Operations'),
        ),
    ]



# Generated by Django 2.2.2 on 2019-06-28 08:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0050_auto_20190625_1220'),
    ]

    operations = [
        migrations.AddField(
            model_name='documents',
            name='load_error',
            field=models.TextField(blank=True, null=True),
        ),
    ]



# Generated by Django 3.0.2 on 2020-01-17 08:11

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0182_auto_20200115_0959'),
    ]

    operations = [
        migrations.RunSQL(
            'update ckk_item_refs set props = 3'
        )
    ]



# Generated by Django 2.2.8 on 2019-12-09 12:55

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0162_auto_20191208_0900'),
    ]

    operations = [
        migrations.AddField(
            model_name='launches',
            name='props',
            field=bitfield.models.BitField((('routing_maked', 'Маршрутизация выполнена'),), db_index=True, default=0),
        ),
    ]



from django.urls import path

from kaf_pas.planing.views import operation_refs

urlpatterns = [

    path('Operation_refs/Fetch/', operation_refs.Operation_refs_Fetch),
    path('Operation_refs/Add', operation_refs.Operation_refs_Add),
    path('Operation_refs/Update', operation_refs.Operation_refs_Update),
    path('Operation_refs/Remove', operation_refs.Operation_refs_Remove),
    path('Operation_refs/Lookup/', operation_refs.Operation_refs_Lookup),
    path('Operation_refs/Info/', operation_refs.Operation_refs_Info),
    path('Operation_refs/Copy', operation_refs.Operation_refs_Copy),

]



# Generated by Django 3.0.2 on 2020-01-27 16:32

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0125_lotsman_documents_hierarcy_path'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='lotsman_documents_hierarcy',
            name='path',
        ),
    ]



# Generated by Django 2.2.3 on 2019-07-02 14:40

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.code_field
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0118_auto_20190701_1712'),
    ]

    operations = [
        migrations.AlterField(
            model_name='item_line',
            name='section',
            field=isc_common.fields.code_field.CodeStrictField(),
        ),
        migrations.AlterField(
            model_name='material_params',
            name='param_type',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='refs.Type_param'),
        ),
    ]



# Generated by Django 2.2.3 on 2019-07-09 02:13

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0050_customer_long_name'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='customer',
            name='full_name',
        ),
    ]



# Generated by Django 2.2.7 on 2019-11-21 21:16

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0159_auto_20191119_2034'),
        ('production', '0132_auto_20191120_1547'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='launch_operations_item',
            unique_together={('item', 'num', 'launch', 'operation')},
        ),
        migrations.AlterUniqueTogether(
            name='operations_item',
            unique_together={('item', 'num', 'operation')},
        ),
    ]



from django.urls import path

from kaf_pas.planing.views import operation_executor

urlpatterns = [

    path('Operation_executor/Fetch/', operation_executor.Operation_executor_Fetch),
    path('Operation_executor/Add', operation_executor.Operation_executor_Add),
    path('Operation_executor/Update', operation_executor.Operation_executor_Update),
    path('Operation_executor/Remove', operation_executor.Operation_executor_Remove),
    path('Operation_executor/Lookup/', operation_executor.Operation_executor_Lookup),
    path('Operation_executor/Info/', operation_executor.Operation_executor_Info),
    path('Operation_executor/Copy', operation_executor.Operation_executor_Copy),

]



# Generated by Django 2.2.7 on 2019-11-22 09:15

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0159_auto_20191119_2034'),
    ]

    operations = [
        migrations.AddField(
            model_name='ed_izm',
            name='props',
            field=bitfield.models.BitField((('root_timevalue', 'Базовая временная еденица'),), db_index=True, default=0),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-20 12:58

from django.db import migrations, models
import django.utils.timezone
import isc_common.fields.code_field
import isc_common.fields.name_field


class Migration(migrations.Migration):

    dependencies = [
        ('input', '0006_delete_operation_plan_types'),
    ]

    operations = [
        migrations.CreateModel(
            name='User_view',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('username', isc_common.fields.code_field.CodeField(verbose_name='логин')),
                ('first_name', isc_common.fields.name_field.NameField(verbose_name='имя')),
                ('last_name', isc_common.fields.name_field.NameField(verbose_name='фамилия')),
                ('email', models.EmailField(blank=True, max_length=254, null=True, verbose_name='E-mail')),
                ('middle_name', isc_common.fields.name_field.NameField(verbose_name='отчетво')),
                ('position_name', isc_common.fields.name_field.NameField(verbose_name='должность')),
                ('location_name', isc_common.fields.name_field.NameField(verbose_name='место работы')),
            ],
            options={
                'verbose_name': 'Пользователи',
                'db_table': 'user_view',
                'managed': False,
            },
        ),
    ]



import logging

from django.core.management import BaseCommand
from django.db import transaction
from tqdm import tqdm

from kaf_pas.ckk.models.item import Item
from kaf_pas.production.models.operations_item import Operations_item

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Тестирование"

    # def add_arguments(self, parser):
    #     parser.add_argument('--demand_id', type=int)
    #     parser.add_argument('--user_id', type=int)

    def handle(self, *args, **options):
        # demand_id = options.get('demand_id')
        # user_id = options.get('user_id')

        no = 0
        with transaction.atomic():
            total = Operations_item.objects.exclude(item__props=Item.props.from_lotsman).count()
            pbar = tqdm(total=total)
            for operations_item in [item for item in Operations_item.objects.exclude(item__props=Item.props.from_lotsman)]:
                if operations_item.item.STMP_1 == None:
                    query = Item.objects.filter(STMP_2__value_str=operations_item.item.STMP_2.value_str, props=Item.props.from_lotsman)
                elif operations_item.item.STMP_2 == None:
                    query = Item.objects.filter(STMP_1__value_str=operations_item.item.STMP_1.value_str, props=Item.props.from_lotsman)
                elif operations_item.item.STMP_1 != None and operations_item.item.STMP_2 != None:
                    query = Item.objects.filter(STMP_1__value_str=operations_item.item.STMP_1.value_str, STMP_2__value_str=operations_item.item.STMP_2.value_str, props=Item.props.from_lotsman)

                for item_lotsman in query:
                    # lotsman_operations_item, created = Operations_item.objects.get_or_create(
                    #     item=item_lotsman,
                    #     operation=operations_item.operation,
                    #     defaults=dict(
                    #         ed_izm=operations_item.ed_izm,
                    #         qty=operations_item.qty,
                    #         num=operations_item.num,
                    #         description=operations_item.description,
                    #     ))
                    #
                    # if created:
                    #     for operation_resources in Operation_resources.objects.filter(operationitem=operations_item):
                    #         Operation_resources.objects.get_or_create(
                    #             operationitem=lotsman_operations_item,
                    #             resource=operation_resources.resource,
                    #             location=operation_resources.location
                    #         )
                    #
                    #     for operation_material in Operation_material.objects.filter(operationitem=operations_item):
                    #         Operation_material.objects.get_or_create(
                    #             operationitem=lotsman_operations_item,
                    #             material=operation_material.material,
                    #             material_askon=operation_material.material_askon,
                    #             ed_izm=operation_material.ed_izm,
                    #             qty=operation_material.qty,
                    #         )
                    no += 1
                pbar.update()
            pbar.close()
        percents = round(no * 100 / total, 2)
        print(f'{no} - {percents}% Done.')



# Generated by Django 3.0.3 on 2020-02-22 17:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0140_auto_20200212_0751'),
    ]

    operations = [
        migrations.AlterField(
            model_name='document_attributes',
            name='value_int',
            field=models.FloatField(blank=True, db_index=True, null=True, verbose_name='Значение атрибута'),
        ),
    ]



# Generated by Django 2.2.2 on 2019-06-13 15:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0047_auto_20190612_2301'),
    ]

    operations = [
        migrations.AlterField(
            model_name='document_attributes',
            name='value_str',
            field=models.TextField(blank=True, db_index=True, null=True, verbose_name='Значение атрибута'),
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-09 11:31

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0056_operation_level'),
    ]

    operations = [
        migrations.AlterField(
            model_name='operation_launches',
            name='launch',
            field=isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, related_name='planing_resource_launch', to='production.Launches'),
        ),
    ]



# Generated by Django 3.0.2 on 2020-01-20 10:25

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0069_auto_20200119_1750'),
    ]

    operations = [
        migrations.CreateModel(
            name='Operations_view',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('value', models.DecimalField(blank=True, decimal_places=4, max_digits=19, null=True)),
                ('description', models.TextField(blank=True, null=True)),
                ('parent_id', models.PositiveIntegerField(blank=True, null=True)),
            ],
            options={
                'verbose_name': 'Опреации системные',
                'db_table': 'planing_operations_view',
                'managed': False,
            },
        ),
    ]



from django.urls import path

from kaf_pas.ckk.views import item_document

urlpatterns = [

    path('Item_document/Fetch/', item_document.Item_document_Fetch),
    path('Item_document/Add', item_document.Item_document_Add),
    path('Item_document/Update', item_document.Item_document_Update),
    path('Item_document/Remove', item_document.Item_document_Remove),
    path('Item_document/Lookup/', item_document.Item_document_Lookup),
    path('Item_document/Info/', item_document.Item_document_Info),
    path('Item_document/Copy', item_document.Item_document_Copy),

]



# Generated by Django 3.0.3 on 2020-02-24 04:13

from django.db import migrations, models
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0110_auto_20200223_1636'),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name='operation_refs',
            name='Operation_refs_unique_constraint_0',
        ),
        migrations.RemoveConstraint(
            model_name='operation_refs',
            name='Operation_refs_unique_constraint_1',
        ),
        migrations.AddField(
            model_name='operation_refs',
            name='parent_real',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='operation_parent_real', to='planing.Operations'),
        ),
        migrations.AddConstraint(
            model_name='operation_refs',
            constraint=models.UniqueConstraint(condition=models.Q(('parent', None), ('parent_real', None)), fields=('child',), name='Operation_refs_unique_constraint_0'),
        ),
        migrations.AddConstraint(
            model_name='operation_refs',
            constraint=models.UniqueConstraint(condition=models.Q(parent_real=None), fields=('child', 'parent'), name='Operation_refs_unique_constraint_1'),
        ),
        migrations.AddConstraint(
            model_name='operation_refs',
            constraint=models.UniqueConstraint(condition=models.Q(parent=None), fields=('child', 'parent_real'), name='Operation_refs_unique_constraint_2'),
        ),
        migrations.AddConstraint(
            model_name='operation_refs',
            constraint=models.UniqueConstraint(fields=('child', 'parent', 'parent_real'), name='Operation_refs_unique_constraint_3'),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-22 12:56

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0022_auto_20190522_1243'),
        ('sales', '0022_auto_20190522_1125'),
    ]

    operations = [
        migrations.AddField(
            model_name='precent',
            name='operation_plan_templates',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='production.Operation_plan_templates'),
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-28 21:34

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0101_auto_20191028_2108'),
    ]

    operations = [
        migrations.AlterField(
            model_name='dogovors',
            name='date',
            field=models.DateTimeField(db_index=True, default=django.utils.timezone.now, verbose_name='Дата'),
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-08 08:12

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0159_auto_20191207_1801'),
    ]

    operations = [
        migrations.AlterField(
            model_name='launch_operation_resources',
            name='location',
            field=isc_common.fields.related.ForeignKeyCascade(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='ckk.Locations'),
        ),
        migrations.AlterField(
            model_name='launch_operation_resources',
            name='resource',
            field=isc_common.fields.related.ForeignKeyCascade(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='production.Resource'),
        ),
    ]



# Generated by Django 2.2.7 on 2019-11-18 21:28

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0158_delete_status'),
        ('production', '0110_auto_20191118_2127'),
    ]

    operations = [
        migrations.AddField(
            model_name='launch_item_refs',
            name='launch',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='production.Launches'),
        ),
        migrations.AlterUniqueTogether(
            name='launch_item_refs',
            unique_together={('parent', 'child', 'launch')},
        ),
    ]



# Generated by Django 2.2.2 on 2019-06-10 01:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0044_document_attr_cross_position_in_document'),
    ]

    operations = [
        migrations.AlterField(
            model_name='document_attr_cross',
            name='position_in_document',
            field=models.PositiveIntegerField(),
        ),
    ]



import os
import urllib.request

from django.http import HttpResponse

from kaf_pas.common.UploadItem import UploadItem
from kaf_pas.sales.models.precent import Precent


def download_present_file(request):
    id = request.GET.get('id')
    obj = Precent.objects.get(id=id)
    item = UploadItem(stored_file_name=os.path.basename(obj.attfile.path), key=obj.key)
    if obj.key:
        path_decrypt = item.decrypt()
    else:
        path_decrypt = item.full_path

    with open(path_decrypt, 'rb') as fh:
        response = HttpResponse(fh.read(), content_type=obj.mime_type)
        # response['Content-Disposition'] = f'attachment; filename="{urllib.request.quote(obj.real_name.encode("utf-8"))}"'
        response['Content-Disposition'] = f'attachment; filename="{urllib.request.quote(obj.real_name)}"'
        response['Content-Length'] = str(os.stat(path_decrypt).st_size)
        fh.close()
        if obj.key:
            os.remove(path_decrypt)
        return response



# Generated by Django 2.2.1 on 2019-05-22 07:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0017_auto_20190522_0746'),
    ]

    operations = [
        migrations.AlterField(
            model_name='precent',
            name='date',
            field=models.DateTimeField(db_index=True, default=None, verbose_name='Дата'),
        ),
        migrations.AlterField(
            model_name='precent',
            name='date_sign',
            field=models.DateTimeField(db_index=True, default=None, verbose_name='Дата подписания'),
        ),
        migrations.AlterUniqueTogether(
            name='precent',
            unique_together={('code', 'type', 'date')},
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-29 15:23

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0212_auto_20200225_1013'),
        ('production', '0223_operation_attr'),
    ]

    operations = [
        migrations.RenameField(
            model_name='operation_attr',
            old_name='attr',
            new_name='attr_type',
        ),
        migrations.AlterUniqueTogether(
            name='operation_attr',
            unique_together={('operation', 'attr_type')},
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-07 08:22

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0192_auto_20200206_1540'),
        ('kd', '0130_auto_20200206_1526'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='document_attributes',
            unique_together={('attr_type', 'value_str')},
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-23 12:22

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0201_auto_20200223_0522'),
    ]

    operations = [
        migrations.AlterField(
            model_name='operations',
            name='props',
            field=bitfield.models.BitField((('assemly', 'Операция комплектации'), ('made_common_form', 'Формировать общий документ')), db_index=True, default=0),
        ),
    ]



# Generated by Django 2.2 on 2019-04-16 16:48

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0022_auto_20190416_1014'),
    ]

    operations = [
        migrations.RenameField(
            model_name='item_line',
            old_name='item_child',
            new_name='child',
        ),
        migrations.RenameField(
            model_name='item_line',
            old_name='item_parent',
            new_name='parent',
        ),
    ]



from django.urls import path

from kaf_pas.kd.views import document_attributes__Version

urlpatterns = [

    path('Document_attributes__Version/Fetch/', document_attributes__Version.Document_attributes__Version_Fetch),
    path('Document_attributes__Version/Add', document_attributes__Version.Document_attributes__Version_Add),
    path('Document_attributes__Version/Update', document_attributes__Version.Document_attributes__Version_Update),
    path('Document_attributes__Version/Remove', document_attributes__Version.Document_attributes__Version_Remove),
    path('Document_attributes__Version/Lookup/', document_attributes__Version.Document_attributes__Version_Lookup),
    path('Document_attributes__Version/Info/', document_attributes__Version.Document_attributes__Version_Info),
    path('Document_attributes__Version/Copy', document_attributes__Version.Document_attributes__Version_Copy),

]



import logging

from django.core.management import BaseCommand

from kaf_pas.planing.models.routing import RoutingManager

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    def handle(self, *args, **options):
        # for item in RoutingManager.make_levels(launch_id=23):
        for item in RoutingManager.make_levels():
            print(item)



from django.urls import path

from kaf_pas.production.views import launches

urlpatterns = [

    path('Launches/Fetch/', launches.Launches_Fetch),
    path('Launches/Add', launches.Launches_Add),
    path('Launches/Update', launches.Launches_Update),
    path('Launches/ReCreate', launches.Launches_ReCreate),
    path('Launches/Remove', launches.Launches_Remove),
    path('Launches/Lookup/', launches.Launches_Lookup),
    path('Launches/Info/', launches.Launches_Info),
    path('Launches/Copy', launches.Launches_Copy),

]



# Generated by Django 3.0.2 on 2020-01-22 16:13

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('ckk', '0186_locations_users_parent'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='locations_users',
            unique_together={('location', 'user')},
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-06 12:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0127_auto_20200131_1121'),
        ('ckk', '0190_auto_20200131_1105'),
    ]

    operations = [
        migrations.AddField(
            model_name='item',
            name='lotsman_type_id',
            field=models.BigIntegerField(blank=True, null=True),
        ),
        migrations.AlterUniqueTogether(
            name='item',
            unique_together={('STMP_1', 'STMP_2', 'version', 'lotsman_type_id')},
        ),
    ]



from django.urls import path

from kaf_pas.planing.views import levels

urlpatterns = [

    path('Levels/Fetch/', levels.Levels_Fetch),
    path('Levels/Add', levels.Levels_Add),
    path('Levels/Update', levels.Levels_Update),
    path('Levels/Remove', levels.Levels_Remove),
    path('Levels/Lookup/', levels.Levels_Lookup),
    path('Levels/Info/', levels.Levels_Info),
    path('Levels/Copy', levels.Levels_Copy),

]



# Generated by Django 3.0.3 on 2020-02-26 08:28

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0208_auto_20200223_1613'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Resource_view',
        ),
        migrations.RemoveField(
            model_name='resource',
            name='props',
        ),
    ]



import requests
from django.conf import settings
from django.http import HttpResponseRedirect
from isc_common.common.mat_views import refresh_mat_view

from isc_common.http.DSRequest import DSRequest
from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.kd.models.documents import Documents
from kaf_pas.kd.models.documents_mview import Documents_mview


@JsonResponseWithException()
def Documents_mview_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Documents_mview.objects.
                select_related('STMP_1', 'STMP_2').
                filter(attr_type__code__in=['KD_PDF', 'CDW', 'SPW', 'LOTSMAN']).
                get_range_rows1(
                request=request,
                function=Documents_mview.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Documents_bad_mview_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Documents_mview.objects.
                select_related('STMP_1', 'STMP_2').
                filter(attr_type__code__in=['KD_BAD']).
                get_range_rows1(
                request=request,
                function=Documents_mview.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Documents_mview_Add(request):
    return JsonResponse(DSResponseAdd(data=Documents.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Documents_mview_Update(request):
    return JsonResponse(DSResponseUpdate(data=Documents.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Documents_mview_Remove(request):
    ws_channel = request.session.get('ws_channel')
    ws_port = settings.WS_PORT
    host = settings.WS_HOST

    _request = DSRequest(request=request)
    ids = _request.get_tuple_ids()
    r = requests.get(f'{settings.KOMPAS_INFORMICA}/logic/Documents/Remove', params=dict(ids=ids, port=ws_port, ws_channel=ws_channel, host=host, user_id=_request.user_id))
    HttpResponseRedirect(r.url)
    return JsonResponse(DSResponse(request=request, status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Documents_mview_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Documents.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Documents_mview_Info(request):
    return JsonResponse(DSResponse(request=request, data=Documents_mview.objects.select_related('STMP_1', 'STMP_2').get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Documents_mview_bad_Info(request):
    return JsonResponse(DSResponse(request=request, data=Documents_mview.objects.select_related('STMP_1', 'STMP_2').filter(attr_type__code__in=['KD_BAD']).get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Documents_mview_RefreshMView(request):
    refresh_mat_view('kd_documents_mview')
    return JsonResponse(DSResponse(request=request, status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 2.2.2 on 2019-06-06 18:39

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0032_auto_20190530_1706'),
    ]

    operations = [
        migrations.AlterModelTable(
            name='documents_view',
            table='kd_documents_view',
        ),
    ]



import logging

from bitfield import BitField
from django.db import transaction
from django.db.models import PositiveIntegerField

from isc_common.fields.related import ForeignKeyProtect, ForeignKeyCascade
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
from kaf_pas.ckk.models.item import Item

logger = logging.getLogger(__name__)


class Item_qtyQuerySet(AuditQuerySet):
    def create(self, **kwargs):
        return super().create(**kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Item_qtyManager(AuditManager):

    @staticmethod
    def get_prop():
        return BitField(flags=(
            ('items', 'Состав изделия (Технологическая спецификация)'),  # 1
            ('teach', 'Производственная спецификация'),  # 2
        ), default=1, db_index=True)

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Item_qtyQuerySet(self.model, using=self._db)


class Item_qty(AuditModel):
    child = ForeignKeyCascade(Item, related_name='Item_qty_child')
    parent = ForeignKeyCascade(Item, related_name='Item_qty_parent', blank=True, null=True)
    qty = PositiveIntegerField()
    props=Item_qtyManager.get_prop()

    objects = Item_qtyManager()

    def __str__(self):
        return f'ID:{self.id}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Количество элементов'



# Generated by Django 2.2.8 on 2019-12-12 20:41

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0172_auto_20191212_2039'),
    ]

    operations = [
        migrations.AlterField(
            model_name='item_refs',
            name='syntetic_child',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='syntetic_child', to='ckk.Item'),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-24 09:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0035_auto_20190524_0932'),
    ]

    operations = [
        migrations.AlterField(
            model_name='precent',
            name='date',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата'),
        ),
    ]



# Generated by Django 2.2.7 on 2019-11-19 18:43

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0124_auto_20191119_1811'),
    ]

    operations = [
        migrations.AlterModelTable(
            name='launch_item_line_view',
            table='production_launch_item_line_view',
        ),
        migrations.AlterModelTable(
            name='launch_item_view',
            table='production_launch_item_view',
        ),
    ]



from django.urls import path

from kaf_pas.production.views import launch_item_line_view

urlpatterns = [

    path('Launch_item_line_view/Fetch/', launch_item_line_view.Launch_item_line_view_Fetch),
    path('Launch_item_line_view/Fetch1/', launch_item_line_view.Launch_item_line_view_Fetch1),
    path('Launch_item_line_view/Add', launch_item_line_view.Launch_item_line_view_Add),
    path('Launch_item_line_view/Update', launch_item_line_view.Launch_item_line_view_Update),
    path('Launch_item_line_view/Remove', launch_item_line_view.Launch_item_line_view_Remove),
    path('Launch_item_line_view/Lookup/', launch_item_line_view.Launch_item_line_view_Lookup),
    path('Launch_item_line_view/Info/', launch_item_line_view.Launch_item_line_view_Info),
    path('Launch_item_line_view/Copy', launch_item_line_view.Launch_item_line_view_Copy),

]



import logging

from bitfield import BitField
from django.db.models import DecimalField, FloatField

from isc_common.fields.related import ForeignKeyCascade, ForeignKeyProtect
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
from kaf_pas.ckk.models.ed_izm import Ed_izm
from kaf_pas.planing.models.operations import Operations

logger = logging.getLogger(__name__)


class Operation_valueQuerySet(AuditQuerySet):
    pass


class Operation_valueManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Operation_valueQuerySet(self.model, using=self._db)


class Operation_value(AuditModel):
    operation = ForeignKeyCascade(Operations)
    edizm = ForeignKeyProtect(Ed_izm)
    value = FloatField()
    props = BitField(flags=(
        ('perone', 'Из расчета на еденицу продукции'),
        ('used_in_release', 'Учавствовало в учете выпуска'),
    ), default=0, db_index=True)

    objects = Operation_valueManager()

    def __str__(self):
        return f"ID:{self.id}, value: {self.value}, props: {self.props}, edizm: [{self.edizm}]"

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Кросс таблица значений операции'



# Generated by Django 2.2.6 on 2019-10-23 22:12

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0090_auto_20191023_2206'),
    ]

    operations = [
        migrations.RenameField(
            model_name='precent_materials',
            old_name='precent',
            new_name='precent_item',
        ),
    ]



# Generated by Django 2.1.7 on 2019-03-27 07:03

import bitfield.models
from django.db import migrations, models
import django.utils.timezone
import isc_common.fields.code_field
import isc_common.fields.description_field
import isc_common.fields.name_field


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Attr_type',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted', models.BooleanField(db_index=True, default=False, verbose_name='Признак удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('description', isc_common.fields.description_field.DescriptionField()),
                ('code', isc_common.fields.code_field.CodeField(unique=True, verbose_name='Код')),
                ('name', isc_common.fields.name_field.NameField(verbose_name='Наименование')),
                ('props', bitfield.models.BitField((('relevant', 'Актуальность'),), default=1)),
            ],
            options={
                'verbose_name': 'Тип атрибута',
            },
        ),
        migrations.CreateModel(
            name='Item',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted', models.BooleanField(db_index=True, default=False, verbose_name='Признак удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('STMP_1', isc_common.fields.name_field.NameField(verbose_name='Наименование изделия')),
                ('STMP_2', isc_common.fields.name_field.NameField(verbose_name='Обозначение документа')),
                ('props', bitfield.models.BitField((('relevant', 'Актуальность'),), default=1)),
            ],
            options={
                'verbose_name': 'Товарная позиция',
            },
        ),
        migrations.CreateModel(
            name='Item_line',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted', models.BooleanField(db_index=True, default=False, verbose_name='Признак удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('section', isc_common.fields.code_field.CodeField(blank=True, null=True, verbose_name='Раздел')),
                ('subsection', isc_common.fields.code_field.CodeField(blank=True, null=True, verbose_name='Подраздел')),
                ('props', bitfield.models.BitField((('relevant', 'Актуальность'), ('relevant1', 'Актуальность1')), default=1)),
            ],
            options={
                'verbose_name': 'Строка состава изделия',
            },
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-21 20:28

from django.db import migrations, models
import isc_common.fields.files
import isc_common.fields.name_field


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0015_merge_20190521_1855'),
    ]

    operations = [
        migrations.AddField(
            model_name='precent',
            name='attfile',
            field=isc_common.fields.files.FileFieldEx(blank=True, default=None, max_length=255, null=True, upload_to='', verbose_name='Файл'),
        ),
        migrations.AddField(
            model_name='precent',
            name='file_store',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='Каталог хранения файла'),
        ),
        migrations.AddField(
            model_name='precent',
            name='format',
            field=isc_common.fields.name_field.NameField(verbose_name='Формат файла'),
        ),
        migrations.AddField(
            model_name='precent',
            name='key',
            field=models.BinaryField(blank=True, max_length=200, null=True),
        ),
        migrations.AddField(
            model_name='precent',
            name='mime_type',
            field=isc_common.fields.name_field.NameField(blank=True, default=None, null=True, verbose_name='MIME тип файла файла'),
        ),
        migrations.AddField(
            model_name='precent',
            name='real_name',
            field=models.TextField(blank=True, default=None, null=True, verbose_name='Первоначальное имя файла'),
        ),
        migrations.AddField(
            model_name='precent',
            name='size',
            field=models.BigIntegerField(default=0, verbose_name='Размер фала'),
        ),
    ]



import logging

from bitfield import BitField
from django.db.models import PositiveIntegerField

from isc_common.fields.related import ForeignKeyProtect
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
from kaf_pas.ckk.models.item import Item
from kaf_pas.ckk.models.item_qty import Item_qtyManager
from kaf_pas.kd.models.document_attributes import Document_attributes

logger = logging.getLogger(__name__)


class Item_qty_viewQuerySet(AuditQuerySet):
    def delete(self):
        return super().delete()

    def create(self, **kwargs):
        return super().create(**kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Item_qty_viewManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'editing': record.editing,
            'deliting': record.deliting,
            'qty': record.qty,
            'lastmodified': record.lastmodified,
            "STMP_1_id": record.STMP_1.id if record.STMP_1 else None,
            "STMP_1__value_str": record.STMP_1.value_str if record.STMP_1 else None,
            "STMP_2_id": record.STMP_2.id if record.STMP_2 else None,
            "STMP_2__value_str": record.STMP_2.value_str if record.STMP_2 else None,
        }
        return res

    def get_queryset(self):
        return Item_qty_viewQuerySet(self.model, using=self._db)


class Item_qty_view(AuditModel):
    STMP_1 = ForeignKeyProtect(Document_attributes, verbose_name='Наименование изделия', related_name='STMP_1_qty_view', null=True, blank=True)
    STMP_2 = ForeignKeyProtect(Document_attributes, verbose_name='Обозначение изделия', related_name='STMP_2_qty_view', null=True, blank=True)
    child = ForeignKeyProtect(Item, related_name='Item_qty_view_child')
    parent = ForeignKeyProtect(Item, related_name='Item_qty_view_parent', blank=True, null=True)
    qty = PositiveIntegerField()
    props = Item_qtyManager.get_prop()

    objects = Item_qty_viewManager()

    def __str__(self):
        return f'ID:{self.id}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Количество элементов'
        managed = False
        db_table = 'ckk_item_qty_view'



# Generated by Django 2.2.8 on 2019-12-07 18:01

from django.db import migrations, models
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0159_auto_20191207_1801'),
        ('ckk', '0163_locations_users'),
        ('planing', '0037_auto_20191207_1312'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='operation_launch_operation_material',
            name='launch_operation_material',
        ),
        migrations.RemoveField(
            model_name='operation_launch_operation_resources',
            name='launch_operation_resources',
        ),
        migrations.RemoveField(
            model_name='operation_launch_operations_item',
            name='launch_operations_item',
        ),
        migrations.AddField(
            model_name='operation_launch_operation_material',
            name='edizm',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='ckk.Ed_izm'),
        ),
        migrations.AddField(
            model_name='operation_launch_operation_material',
            name='material',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='ckk.Materials'),
        ),
        migrations.AddField(
            model_name='operation_launch_operation_material',
            name='material_askon',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='ckk.Material_askon'),
        ),
        migrations.AddField(
            model_name='operation_launch_operation_material',
            name='qty',
            field=models.DecimalField(decimal_places=4, default=0.0, max_digits=10),
        ),
        migrations.AddField(
            model_name='operation_launch_operation_resources',
            name='location',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='ckk.Locations'),
        ),
        migrations.AddField(
            model_name='operation_launch_operation_resources',
            name='resource',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='production.Resource'),
        ),
        migrations.AddField(
            model_name='operation_launch_operations_item',
            name='ed_izm',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='ckk.Ed_izm'),
        ),
        migrations.AddField(
            model_name='operation_launch_operations_item',
            name='num',
            field=models.PositiveIntegerField(default=None),
        ),
        migrations.AddField(
            model_name='operation_launch_operations_item',
            name='qty',
            field=models.PositiveIntegerField(default=None),
        ),
        migrations.AlterField(
            model_name='operation_launch_operations_item',
            name='operation',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='planing.Operations'),
        ),
        migrations.AlterField(
            model_name='operations',
            name='value',
            field=models.DecimalField(blank=True, decimal_places=4, max_digits=19, null=True),
        ),
    ]



import logging

from django.db.models import BigIntegerField, BooleanField, PositiveIntegerField

from isc_common.common.mat_views import create_view
from isc_common.datetime import DateToStr
from isc_common.fields.code_field import CodeField
from isc_common.fields.name_field import NameField
from isc_common.fields.related import ForeignKeyProtect
from isc_common.models.audit import AuditQuerySet, AuditManager
from isc_common.models.base_ref import Hierarcy
from isc_common.number import DelProps
from kaf_pas.ckk.models.item import Item_add
from kaf_pas.ckk.models.item_line import Item_lineManager
from kaf_pas.ckk.models.item_refs import Item_refsManager
from kaf_pas.kd.models.document_attributes import Document_attributes
from kaf_pas.kd.models.documents import Documents

logger = logging.getLogger(__name__)


class Tmp_Item_operations_viewQuerySet(AuditQuerySet):
    pass


class Tmp_Item_operations_viewManager(AuditManager):
    @staticmethod
    def create():
        create_view(
            view_name='ckk_tmp_item_operations_view',
            sql_str='''SELECT ckk_item.id,
                               ckk_item.editing,
                               ckk_item.deliting,
                               ckk_item.lastmodified,
                               ckk_item."STMP_1_id",
                               ckk_item."STMP_2_id",
                               ckk_item.props,
                               ckk_item.document_id,
                               ckk_item.deleted_at,
                               ckk_item.parent_id,
                               ckk_item.child_id,
                               ckk_item.relevant,
                               ckk_item.confirmed,
                               CASE
                                   WHEN ckk_item.from_cdw IS NOT NULL THEN ckk_item.from_cdw
                                   WHEN ckk_item.from_spw IS NOT NULL THEN ckk_item.from_spw
                                   WHEN ckk_item.from_pdf IS NOT NULL THEN ckk_item.from_pdf
                                   WHEN ckk_item.man_input IS NOT NULL THEN ckk_item.man_input
                                   WHEN ckk_item.copied IS NOT NULL THEN ckk_item.copied
                                   WHEN ckk_item.on_activated IS NOT NULL THEN ckk_item.on_activated
                                   WHEN ckk_item.from_lotsman IS NOT NULL THEN ckk_item.from_lotsman
                                   ELSE NULL::text
                                   END AS where_from,
                               ckk_item.for_line,
                               ckk_item."isFolder",
                               ckk_item.version,
                               ckk_item.qty_operations,
                               ckk_item.section,
                               ckk_item.refs_props,
                               ckk_item.refs_id
                        FROM (SELECT ckk_item_1.id,
                                     ckk_item_1.editing,
                                     ckk_item_1.deliting,
                                     ckk_item_1.lastmodified,
                                     ckk_item_1."STMP_1_id",
                                     ckk_item_1."STMP_2_id",
                                     ckk_item_1.props,
                                     ckk_item_1.document_id,
                                     ckk_item_1.deleted_at,
                                     ckk_item_1.version,
                                     refs1.id                                   AS refs_id,
                                     refs1.parent_id,
                                     refs1.props                                AS refs_props,
                                     (SELECT DISTINCT ckk_item_line.section
                                      FROM ckk_item_line
                                      WHERE ckk_item_line.parent_id = refs1.parent_id
                                        AND ckk_item_line.child_id = refs1.child_id
                                      LIMIT 1)                                  AS section,
                                     refs1.child_id,
                                     CASE
                                         WHEN ((SELECT count(1) AS count
                                                FROM ckk_item_refs
                                                WHERE ckk_item_refs.parent_id = ckk_item_1.id)) > 0 THEN true
                                         ELSE false
                                         END                                    AS "isFolder",
                                     relevant_column(ckk_item_1.props::integer) AS relevant,
                                     confirmed_column(ckk_item_1.props::integer) AS confirmed,
                                     CASE
                                         WHEN is_bit_on(ckk_item_1.props::integer, 0) THEN 'Первоначально создано'::text
                                         ELSE NULL::text
                                         END                                    AS on_activated,
                                     CASE
                                         WHEN is_bit_on(ckk_item_1.props::integer, 1) THEN 'Получено из чертежа'::text
                                         ELSE NULL::text
                                         END                                    AS from_cdw,
                                     CASE
                                         WHEN is_bit_on(ckk_item_1.props::integer, 2) THEN 'Получено из спецификации'::text
                                         ELSE NULL::text
                                         END                                    AS from_spw,
                                     CASE
                                         WHEN is_bit_on(ckk_item_1.props::integer, 3) THEN 'Строка спецификации'::text
                                         ELSE NULL::text
                                         END                                    AS for_line,
                                     CASE
                                         WHEN is_bit_on(ckk_item_1.props::integer, 4) THEN 'Получено из бумажного архива'::text
                                         ELSE NULL::text
                                         END                                    AS from_pdf,
                                     CASE
                                         WHEN is_bit_on(ckk_item_1.props::integer, 5) THEN 'Занесено вручную'::text
                                         ELSE NULL::text
                                         END                                    AS man_input,
                                     CASE
                                         WHEN is_bit_on(ckk_item_1.props::integer, 6) THEN 'Скопировано'::text
                                         ELSE NULL::text
                                         END                                    AS copied,
                                     CASE
                                         WHEN is_bit_on(ckk_item_1.props::integer, 7) THEN 'Получено из Лоцмана'::text
                                         ELSE NULL::text
                                         END                                    AS from_lotsman,
                                     (SELECT count(*) AS count
                                      FROM production_operations_item poi
                                      WHERE poi.item_id = ckk_item_1.id)        AS qty_operations
                              FROM ckk_item ckk_item_1
                                       JOIN ckk_tmp_item_refs refs1 ON ckk_item_1.id = refs1.child_id) ckk_item'''
        )

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'refs_id': record.refs_id,
            'STMP_1_id': record.STMP_1.id if record.STMP_1 else None,
            'STMP_1__value_str': record.STMP_1.value_str if record.STMP_1 else None,
            'STMP_2_id': record.STMP_2.id if record.STMP_2 else None,
            'STMP_2__value_str': record.STMP_2.value_str if record.STMP_2 else None,
            'lastmodified': DateToStr(record.lastmodified, mask='%Y-%m-%dT%H:%M:%S'),
            'document_id': record.document.id if record.document else None,
            'document__file_document': record.document.file_document if record.document else None,
            'parent_id': record.parent_id,
            'editing': record.editing,
            'deliting': record.deliting,
            'isFolder': record.isFolder,
            'relevant': record.relevant,
            'confirmed': record.confirmed,
            'version': record.version,
            'section': record.section,
            'where_from': record.where_from,
            'qty_operations': record.qty_operations,
            'icon': Item_lineManager.getIcon(record),
            'props': record.props,
            'used': record.refs_props.used,
            'refs_props': record.refs_props,
        }
        # print(res)
        return DelProps(res)

    def get_queryset(self):
        return Tmp_Item_operations_viewQuerySet(self.model, using=self._db)


class Tmp_Item_operations_view(Hierarcy):
    STMP_1 = ForeignKeyProtect(Document_attributes, verbose_name='Наименование изделия', related_name='tmp_STMP_1_view_operations', null=True, blank=True)
    STMP_2 = ForeignKeyProtect(Document_attributes, verbose_name='Обозначение изделия', related_name='tmp_STMP_2_view_operations', null=True, blank=True)
    document = ForeignKeyProtect(Documents, verbose_name='Документ', null=True, blank=True)
    refs_id = BigIntegerField()
    relevant = NameField()
    confirmed = NameField()
    where_from = NameField()
    props = Item_add.get_prop_field()
    refs_props = Item_refsManager.props()
    version = PositiveIntegerField(null=True, blank=True)
    qty_operations = PositiveIntegerField()
    section = CodeField(null=True, blank=True)

    isFolder = BooleanField()

    objects = Tmp_Item_operations_viewManager()

    @property
    def item(self):
        from kaf_pas.ckk.models.item import Item
        return Item.objects.get(id=self.id)

    def __str__(self):
        return f'ID={self.id} STMP_1=[{self.STMP_1}], STMP_2=[{self.STMP_2}], props={self.props}'

    class Meta:
        managed = False
        db_table = 'ckk_tmp_item_operations_view'
        verbose_name = 'Товарная позиция'



from django.urls import path

from kaf_pas.kd.views import cdw_attrs

urlpatterns = [

    path('Cdw_attrs/Fetch/', cdw_attrs.Cdw_attrs_Fetch),
    path('Cdw_attrs/Add', cdw_attrs.Cdw_attrs_Add),
    path('Cdw_attrs/Update', cdw_attrs.Cdw_attrs_Update),
    path('Cdw_attrs/Remove', cdw_attrs.Cdw_attrs_Remove),
    path('Cdw_attrs/Lookup/', cdw_attrs.Cdw_attrs_Lookup),
    path('Cdw_attrs/Info/', cdw_attrs.Cdw_attrs_Info),

]



# Generated by Django 2.2 on 2019-04-14 04:29

from django.db import migrations
import django.db.models.manager


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0009_auto_20190413_1750'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='documents_thumb10',
            managers=[
                ('object', django.db.models.manager.Manager()),
            ],
        ),
    ]



# Generated by Django 2.2.3 on 2019-07-31 22:27

from django.db import migrations, models
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0052_auto_20190709_1032'),
    ]

    operations = [
        migrations.AlterField(
            model_name='operations_item',
            name='ed_izm',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, default=None, null=True, on_delete=django.db.models.deletion.PROTECT, to='ckk.Ed_izm'),
        ),
        migrations.AlterField(
            model_name='operations_item',
            name='qty',
            field=models.PositiveIntegerField(blank=True, default=None, null=True),
        ),
    ]



# Generated by Django 2.2 on 2019-04-13 15:24

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0007_auto_20190413_1215'),
    ]

    operations = [
        migrations.AlterField(
            model_name='documents',
            name='props',
            field=bitfield.models.BitField((('relevant', 'Актуальность'), ('beenItemed', 'Был внесен в состав изделий')), default=1),
        ),
    ]



import logging

from isc_common.models.audit import AuditQuerySet, AuditManager
from isc_common.models.base_ref import BaseRef

logger = logging.getLogger(__name__)


class User_positionsQuerySet(AuditQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class User_positionsManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'code': record.code,
            'name': record.name,
            'description': record.description,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return User_positionsQuerySet(self.model, using=self._db)


class User_positions(BaseRef):
    objects = User_positionsManager()

    def __str__(self):
        return f"{self.id}"

    class Meta:
        verbose_name = 'Должности'



from django.urls import path

from kaf_pas.kd.views import uploades_log

urlpatterns = [

    path('Uploades_log/Fetch/', uploades_log.Uploades_log_Fetch),
    path('Uploades_log/Add', uploades_log.Uploades_log_Add),
    path('Uploades_log/Update', uploades_log.Uploades_log_Update),
    path('Uploades_log/Remove', uploades_log.Uploades_log_Remove),
    path('Uploades_log/Lookup/', uploades_log.Uploades_log_Lookup),
    path('Uploades_log/Info/', uploades_log.Uploades_log_Info),
    path('Uploades_log/Copy', uploades_log.Uploades_log_Copy),

]



# Generated by Django 3.0.3 on 2020-02-19 14:46

from django.db import migrations
import isc_common.fields.code_field


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0101_auto_20200219_0500'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='operations_view',
            name='location',
        ),
        migrations.AddField(
            model_name='operations_view',
            name='mark',
            field=isc_common.fields.code_field.CodeField(blank=True, null=True),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-24 09:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0028_auto_20190523_1753'),
    ]

    operations = [
        migrations.AlterField(
            model_name='customer',
            name='description',
            field=models.TextField(blank=True, null=True, verbose_name='Описание'),
        ),
        migrations.AlterField(
            model_name='customer',
            name='full_name',
            field=models.TextField(blank=True, null=True, verbose_name='Полное наименование'),
        ),
        migrations.AlterField(
            model_name='customer',
            name='name',
            field=models.CharField(db_index=True, max_length=255, verbose_name='Наименование'),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-28 21:55

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0062_auto_20190528_1534'),
        ('sales', '0043_auto_20190528_2049'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Precent_items_view',
        ),
        migrations.AddField(
            model_name='precent_items',
            name='location',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='ckk.Locations'),
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-27 08:19

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import isc_common.fields.code_field
import isc_common.fields.description_field
import isc_common.fields.name_field
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0212_auto_20200226_1412'),
    ]

    operations = [
        migrations.CreateModel(
            name='Operations_template',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('code', isc_common.fields.code_field.CodeField(blank=True, null=True)),
                ('name', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('description', isc_common.fields.description_field.DescriptionField()),
            ],
            options={
                'verbose_name': 'Шаблоны группы операций',
            },
        ),
        migrations.CreateModel(
            name='Operations_template_detail',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
            ],
            options={
                'verbose_name': 'Детализация шаблона группы оперций',
            },
        ),
        # migrations.RemoveConstraint(
        #     model_name='resource',
        #     name='Resource_unique_constraint_0',
        # ),
        # migrations.RemoveConstraint(
        #     model_name='resource',
        #     name='Resource_unique_constraint_1',
        # ),
        migrations.AddField(
            model_name='operations_template_detail',
            name='operation',
            field=isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='production.Operations'),
        ),
        migrations.AddField(
            model_name='operations_template_detail',
            name='template',
            field=isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='production.Operations_template'),
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-20 12:35

from django.db import migrations, models
import isc_common.fields.name_field


class Migration(migrations.Migration):

    dependencies = [
        ('input', '0007_user_view'),
    ]

    operations = [
        migrations.AddField(
            model_name='user_add_info',
            name='comp_name',
            field=isc_common.fields.name_field.NameField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='user_add_info',
            name='ip',
            field=models.IPAddressField(blank=True, null=True),
        ),
    ]
